# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import uuid

from openstack import exceptions
from openstack.network.v2 import _proxy
from openstack.network.v2 import address_group
from openstack.network.v2 import address_scope
from openstack.network.v2 import agent
from openstack.network.v2 import auto_allocated_topology
from openstack.network.v2 import availability_zone
from openstack.network.v2 import bgp_peer
from openstack.network.v2 import bgp_speaker
from openstack.network.v2 import bgpvpn
from openstack.network.v2 import bgpvpn_network_association
from openstack.network.v2 import bgpvpn_port_association
from openstack.network.v2 import bgpvpn_router_association
from openstack.network.v2 import extension
from openstack.network.v2 import firewall_group
from openstack.network.v2 import firewall_policy
from openstack.network.v2 import firewall_rule
from openstack.network.v2 import flavor
from openstack.network.v2 import floating_ip
from openstack.network.v2 import health_monitor
from openstack.network.v2 import l3_conntrack_helper
from openstack.network.v2 import listener
from openstack.network.v2 import load_balancer
from openstack.network.v2 import local_ip
from openstack.network.v2 import local_ip_association
from openstack.network.v2 import metering_label
from openstack.network.v2 import metering_label_rule
from openstack.network.v2 import ndp_proxy
from openstack.network.v2 import network
from openstack.network.v2 import network_ip_availability
from openstack.network.v2 import network_segment_range
from openstack.network.v2 import pool
from openstack.network.v2 import pool_member
from openstack.network.v2 import port
from openstack.network.v2 import port_binding
from openstack.network.v2 import port_forwarding
from openstack.network.v2 import qos_bandwidth_limit_rule
from openstack.network.v2 import qos_dscp_marking_rule
from openstack.network.v2 import qos_minimum_bandwidth_rule
from openstack.network.v2 import qos_minimum_packet_rate_rule
from openstack.network.v2 import qos_packet_rate_limit_rule
from openstack.network.v2 import qos_policy
from openstack.network.v2 import qos_rule_type
from openstack.network.v2 import quota
from openstack.network.v2 import rbac_policy
from openstack.network.v2 import router
from openstack.network.v2 import security_group
from openstack.network.v2 import security_group_rule
from openstack.network.v2 import security_groups_default_statefulness
from openstack.network.v2 import segment
from openstack.network.v2 import service_profile
from openstack.network.v2 import service_provider
from openstack.network.v2 import subnet
from openstack.network.v2 import subnet_pool
from openstack.network.v2 import tap_mirror
from openstack.network.v2 import vpn_endpoint_group
from openstack.network.v2 import vpn_ike_policy
from openstack.network.v2 import vpn_ipsec_policy
from openstack.network.v2 import vpn_ipsec_site_connection
from openstack.network.v2 import vpn_service
from openstack import proxy as proxy_base
from openstack.tests.unit import test_proxy_base


QOS_POLICY_ID = 'qos-policy-id-' + uuid.uuid4().hex
QOS_RULE_ID = 'qos-rule-id-' + uuid.uuid4().hex
NETWORK_ID = 'network-id-' + uuid.uuid4().hex
AGENT_ID = 'agent-id-' + uuid.uuid4().hex
ROUTER_ID = 'router-id-' + uuid.uuid4().hex
FIP_ID = 'fip-id-' + uuid.uuid4().hex
CT_HELPER_ID = 'ct-helper-id-' + uuid.uuid4().hex
LOCAL_IP_ID = 'lip-id-' + uuid.uuid4().hex
BGPVPN_ID = 'bgpvpn-id-' + uuid.uuid4().hex
PORT_ID = 'port-id-' + uuid.uuid4().hex


class TestNetworkProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)

    def verify_update(
        self,
        test_method,
        resource_type,
        base_path=None,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        expected_result="result",
        mock_method="openstack.network.v2._proxy.Proxy._update",
    ):
        super().verify_update(
            test_method,
            resource_type,
            base_path=base_path,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=expected_args,
            expected_kwargs=expected_kwargs,
            expected_result=expected_result,
            mock_method=mock_method,
        )

    def verify_delete(
        self,
        test_method,
        resource_type,
        ignore_missing=True,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        mock_method="openstack.network.v2._proxy.Proxy._delete",
    ):
        super().verify_delete(
            test_method,
            resource_type,
            ignore_missing=ignore_missing,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=expected_args,
            expected_kwargs=expected_kwargs,
            mock_method=mock_method,
        )


class TestNetworkAddressGroup(TestNetworkProxy):
    def test_address_group_create_attrs(self):
        self.verify_create(
            self.proxy.create_address_group, address_group.AddressGroup
        )

    def test_address_group_delete(self):
        self.verify_delete(
            self.proxy.delete_address_group, address_group.AddressGroup, False
        )

    def test_address_group_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_address_group, address_group.AddressGroup, True
        )

    def test_address_group_find(self):
        self.verify_find(
            self.proxy.find_address_group, address_group.AddressGroup
        )

    def test_address_group_get(self):
        self.verify_get(
            self.proxy.get_address_group, address_group.AddressGroup
        )

    def test_address_groups(self):
        self.verify_list(self.proxy.address_groups, address_group.AddressGroup)

    def test_address_group_update(self):
        self.verify_update(
            self.proxy.update_address_group, address_group.AddressGroup
        )

    @mock.patch(
        'openstack.network.v2._proxy.Proxy.add_addresses_to_address_group'
    )
    def test_add_addresses_to_address_group(self, add_addresses):
        data = mock.sentinel

        self.proxy.add_addresses_to_address_group(
            address_group.AddressGroup, data
        )

        add_addresses.assert_called_once_with(address_group.AddressGroup, data)

    @mock.patch(
        'openstack.network.v2._proxy.Proxy.remove_addresses_from_address_group'
    )
    def test_remove_addresses_from_address_group(self, remove_addresses):
        data = mock.sentinel

        self.proxy.remove_addresses_from_address_group(
            address_group.AddressGroup, data
        )

        remove_addresses.assert_called_once_with(
            address_group.AddressGroup, data
        )


class TestNetworkAddressScope(TestNetworkProxy):
    def test_address_scope_create_attrs(self):
        self.verify_create(
            self.proxy.create_address_scope, address_scope.AddressScope
        )

    def test_address_scope_delete(self):
        self.verify_delete(
            self.proxy.delete_address_scope, address_scope.AddressScope, False
        )

    def test_address_scope_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_address_scope, address_scope.AddressScope, True
        )

    def test_address_scope_find(self):
        self.verify_find(
            self.proxy.find_address_scope, address_scope.AddressScope
        )

    def test_address_scope_get(self):
        self.verify_get(
            self.proxy.get_address_scope, address_scope.AddressScope
        )

    def test_address_scopes(self):
        self.verify_list(self.proxy.address_scopes, address_scope.AddressScope)

    def test_address_scope_update(self):
        self.verify_update(
            self.proxy.update_address_scope, address_scope.AddressScope
        )


class TestNetworkAgent(TestNetworkProxy):
    def test_agent_delete(self):
        self.verify_delete(self.proxy.delete_agent, agent.Agent, True)

    def test_agent_get(self):
        self.verify_get(self.proxy.get_agent, agent.Agent)

    def test_agents(self):
        self.verify_list(self.proxy.agents, agent.Agent)

    def test_agent_update(self):
        self.verify_update(self.proxy.update_agent, agent.Agent)


class TestNetworkAvailability(TestNetworkProxy):
    def test_availability_zones(self):
        self.verify_list(
            self.proxy.availability_zones, availability_zone.AvailabilityZone
        )

    def test_dhcp_agent_hosting_networks(self):
        self.verify_list(
            self.proxy.dhcp_agent_hosting_networks,
            network.DHCPAgentHostingNetwork,
            method_kwargs={'agent': AGENT_ID},
            expected_kwargs={'agent_id': AGENT_ID},
        )

    def test_network_hosting_dhcp_agents(self):
        self.verify_list(
            self.proxy.network_hosting_dhcp_agents,
            agent.NetworkHostingDHCPAgent,
            method_kwargs={'network': NETWORK_ID},
            expected_kwargs={'network_id': NETWORK_ID},
        )


class TestNetworkExtension(TestNetworkProxy):
    def test_extension_find(self):
        self.verify_find(self.proxy.find_extension, extension.Extension)

    def test_extensions(self):
        self.verify_list(self.proxy.extensions, extension.Extension)

    def test_floating_ip_create_attrs(self):
        self.verify_create(self.proxy.create_ip, floating_ip.FloatingIP)

    def test_floating_ip_delete(self):
        self.verify_delete(
            self.proxy.delete_ip,
            floating_ip.FloatingIP,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_floating_ip_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_ip,
            floating_ip.FloatingIP,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_floating_ip_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_ip,
            floating_ip.FloatingIP,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_floating_ip_find(self):
        self.verify_find(self.proxy.find_ip, floating_ip.FloatingIP)

    def test_floating_ip_get(self):
        self.verify_get(self.proxy.get_ip, floating_ip.FloatingIP)

    def test_ips(self):
        self.verify_list(self.proxy.ips, floating_ip.FloatingIP)

    def test_floating_ip_update(self):
        self.verify_update(
            self.proxy.update_ip,
            floating_ip.FloatingIP,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_floating_ip_update_if_revision(self):
        self.verify_update(
            self.proxy.update_ip,
            floating_ip.FloatingIP,
            method_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
        )


class TestNetworkHealthMonitor(TestNetworkProxy):
    def test_health_monitor_create_attrs(self):
        self.verify_create(
            self.proxy.create_health_monitor, health_monitor.HealthMonitor
        )

    def test_health_monitor_delete(self):
        self.verify_delete(
            self.proxy.delete_health_monitor,
            health_monitor.HealthMonitor,
            False,
        )

    def test_health_monitor_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_health_monitor,
            health_monitor.HealthMonitor,
            True,
        )

    def test_health_monitor_find(self):
        self.verify_find(
            self.proxy.find_health_monitor, health_monitor.HealthMonitor
        )

    def test_health_monitor_get(self):
        self.verify_get(
            self.proxy.get_health_monitor, health_monitor.HealthMonitor
        )

    def test_health_monitors(self):
        self.verify_list(
            self.proxy.health_monitors, health_monitor.HealthMonitor
        )

    def test_health_monitor_update(self):
        self.verify_update(
            self.proxy.update_health_monitor, health_monitor.HealthMonitor
        )


class TestNetworkListener(TestNetworkProxy):
    def test_listener_create_attrs(self):
        self.verify_create(self.proxy.create_listener, listener.Listener)

    def test_listener_delete(self):
        self.verify_delete(
            self.proxy.delete_listener, listener.Listener, False
        )

    def test_listener_delete_ignore(self):
        self.verify_delete(self.proxy.delete_listener, listener.Listener, True)

    def test_listener_find(self):
        self.verify_find(self.proxy.find_listener, listener.Listener)

    def test_listener_get(self):
        self.verify_get(self.proxy.get_listener, listener.Listener)

    def test_listeners(self):
        self.verify_list(self.proxy.listeners, listener.Listener)

    def test_listener_update(self):
        self.verify_update(self.proxy.update_listener, listener.Listener)


class TestNetworkLoadBalancer(TestNetworkProxy):
    def test_load_balancer_create_attrs(self):
        self.verify_create(
            self.proxy.create_load_balancer, load_balancer.LoadBalancer
        )

    def test_load_balancer_delete(self):
        self.verify_delete(
            self.proxy.delete_load_balancer, load_balancer.LoadBalancer, False
        )

    def test_load_balancer_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_load_balancer, load_balancer.LoadBalancer, True
        )

    def test_load_balancer_find(self):
        self.verify_find(
            self.proxy.find_load_balancer, load_balancer.LoadBalancer
        )

    def test_load_balancer_get(self):
        self.verify_get(
            self.proxy.get_load_balancer, load_balancer.LoadBalancer
        )

    def test_load_balancers(self):
        self.verify_list(self.proxy.load_balancers, load_balancer.LoadBalancer)

    def test_load_balancer_update(self):
        self.verify_update(
            self.proxy.update_load_balancer, load_balancer.LoadBalancer
        )


class TestNetworkMeteringLabel(TestNetworkProxy):
    def test_metering_label_create_attrs(self):
        self.verify_create(
            self.proxy.create_metering_label, metering_label.MeteringLabel
        )

    def test_metering_label_delete(self):
        self.verify_delete(
            self.proxy.delete_metering_label,
            metering_label.MeteringLabel,
            False,
        )

    def test_metering_label_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_metering_label,
            metering_label.MeteringLabel,
            True,
        )

    def test_metering_label_find(self):
        self.verify_find(
            self.proxy.find_metering_label, metering_label.MeteringLabel
        )

    def test_metering_label_get(self):
        self.verify_get(
            self.proxy.get_metering_label, metering_label.MeteringLabel
        )

    def test_metering_labels(self):
        self.verify_list(
            self.proxy.metering_labels, metering_label.MeteringLabel
        )

    def test_metering_label_update(self):
        self.verify_update(
            self.proxy.update_metering_label, metering_label.MeteringLabel
        )

    def test_metering_label_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_metering_label_rule,
            metering_label_rule.MeteringLabelRule,
        )

    def test_metering_label_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_metering_label_rule,
            metering_label_rule.MeteringLabelRule,
            False,
        )

    def test_metering_label_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_metering_label_rule,
            metering_label_rule.MeteringLabelRule,
            True,
        )

    def test_metering_label_rule_find(self):
        self.verify_find(
            self.proxy.find_metering_label_rule,
            metering_label_rule.MeteringLabelRule,
        )

    def test_metering_label_rule_get(self):
        self.verify_get(
            self.proxy.get_metering_label_rule,
            metering_label_rule.MeteringLabelRule,
        )

    def test_metering_label_rules(self):
        self.verify_list(
            self.proxy.metering_label_rules,
            metering_label_rule.MeteringLabelRule,
        )

    def test_metering_label_rule_update(self):
        self.verify_update(
            self.proxy.update_metering_label_rule,
            metering_label_rule.MeteringLabelRule,
        )


class TestNetworkNetwork(TestNetworkProxy):
    def test_network_create_attrs(self):
        self.verify_create(self.proxy.create_network, network.Network)

    def test_network_delete(self):
        self.verify_delete(
            self.proxy.delete_network,
            network.Network,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_network_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_network,
            network.Network,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_network_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_network,
            network.Network,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_network_find(self):
        self.verify_find(self.proxy.find_network, network.Network)

    def test_network_find_with_filter(self):
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_network,
            method_args=["net1"],
            method_kwargs={"project_id": "1"},
            expected_args=[network.Network, "net1"],
            expected_kwargs={"project_id": "1", "ignore_missing": True},
        )

    def test_network_get(self):
        self.verify_get(self.proxy.get_network, network.Network)

    def test_networks(self):
        self.verify_list(self.proxy.networks, network.Network)

    def test_network_update(self):
        self.verify_update(
            self.proxy.update_network,
            network.Network,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_network_update_if_revision(self):
        self.verify_update(
            self.proxy.update_network,
            network.Network,
            method_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
        )


class TestNetworkFlavor(TestNetworkProxy):
    def test_flavor_create_attrs(self):
        self.verify_create(self.proxy.create_flavor, flavor.Flavor)

    def test_flavor_delete(self):
        self.verify_delete(self.proxy.delete_flavor, flavor.Flavor, True)

    def test_flavor_find(self):
        self.verify_find(self.proxy.find_flavor, flavor.Flavor)

    def test_flavor_get(self):
        self.verify_get(self.proxy.get_flavor, flavor.Flavor)

    def test_flavor_update(self):
        self.verify_update(self.proxy.update_flavor, flavor.Flavor)

    def test_flavors(self):
        self.verify_list(self.proxy.flavors, flavor.Flavor)


class TestNetworkLocalIp(TestNetworkProxy):
    def test_local_ip_create_attrs(self):
        self.verify_create(self.proxy.create_local_ip, local_ip.LocalIP)

    def test_local_ip_delete(self):
        self.verify_delete(
            self.proxy.delete_local_ip,
            local_ip.LocalIP,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_local_ip_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_local_ip,
            local_ip.LocalIP,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_local_ip_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_local_ip,
            local_ip.LocalIP,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_local_ip_find(self):
        self.verify_find(self.proxy.find_local_ip, local_ip.LocalIP)

    def test_local_ip_get(self):
        self.verify_get(self.proxy.get_local_ip, local_ip.LocalIP)

    def test_local_ips(self):
        self.verify_list(self.proxy.local_ips, local_ip.LocalIP)

    def test_local_ip_update(self):
        self.verify_update(
            self.proxy.update_local_ip,
            local_ip.LocalIP,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_local_ip_update_if_revision(self):
        self.verify_update(
            self.proxy.update_local_ip,
            local_ip.LocalIP,
            method_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
        )


class TestNetworkLocalIpAssociation(TestNetworkProxy):
    def test_local_ip_association_create_attrs(self):
        self.verify_create(
            self.proxy.create_local_ip_association,
            local_ip_association.LocalIPAssociation,
            method_kwargs={'local_ip': LOCAL_IP_ID},
            expected_kwargs={'local_ip_id': LOCAL_IP_ID},
        )

    def test_local_ip_association_delete(self):
        self.verify_delete(
            self.proxy.delete_local_ip_association,
            local_ip_association.LocalIPAssociation,
            ignore_missing=False,
            method_args=[LOCAL_IP_ID, "resource_or_id"],
            expected_args=["resource_or_id"],
            expected_kwargs={'if_revision': None, 'local_ip_id': LOCAL_IP_ID},
        )

    def test_local_ip_association_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_local_ip_association,
            local_ip_association.LocalIPAssociation,
            ignore_missing=True,
            method_args=[LOCAL_IP_ID, "resource_or_id"],
            expected_args=["resource_or_id"],
            expected_kwargs={'if_revision': None, 'local_ip_id': LOCAL_IP_ID},
        )

    def test_local_ip_association_find(self):
        lip = local_ip.LocalIP.new(id=LOCAL_IP_ID)

        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_local_ip_association,
            method_args=['local_ip_association_id', lip],
            expected_args=[
                local_ip_association.LocalIPAssociation,
                'local_ip_association_id',
            ],
            expected_kwargs={
                'ignore_missing': True,
                'local_ip_id': LOCAL_IP_ID,
            },
        )

    def test_local_ip_association_get(self):
        lip = local_ip.LocalIP.new(id=LOCAL_IP_ID)

        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_local_ip_association,
            method_args=['local_ip_association_id', lip],
            expected_args=[
                local_ip_association.LocalIPAssociation,
                'local_ip_association_id',
            ],
            expected_kwargs={'local_ip_id': LOCAL_IP_ID},
        )

    def test_local_ip_associations(self):
        self.verify_list(
            self.proxy.local_ip_associations,
            local_ip_association.LocalIPAssociation,
            method_kwargs={'local_ip': LOCAL_IP_ID},
            expected_kwargs={'local_ip_id': LOCAL_IP_ID},
        )


class TestNetworkServiceProfile(TestNetworkProxy):
    def test_service_profile_create_attrs(self):
        self.verify_create(
            self.proxy.create_service_profile, service_profile.ServiceProfile
        )

    def test_service_profile_delete(self):
        self.verify_delete(
            self.proxy.delete_service_profile,
            service_profile.ServiceProfile,
            True,
        )

    def test_service_profile_find(self):
        self.verify_find(
            self.proxy.find_service_profile, service_profile.ServiceProfile
        )

    def test_service_profile_get(self):
        self.verify_get(
            self.proxy.get_service_profile, service_profile.ServiceProfile
        )

    def test_service_profiles(self):
        self.verify_list(
            self.proxy.service_profiles, service_profile.ServiceProfile
        )

    def test_service_profile_update(self):
        self.verify_update(
            self.proxy.update_service_profile, service_profile.ServiceProfile
        )


class TestNetworkIpAvailability(TestNetworkProxy):
    def test_network_ip_availability_find(self):
        self.verify_find(
            self.proxy.find_network_ip_availability,
            network_ip_availability.NetworkIPAvailability,
        )

    def test_network_ip_availability_get(self):
        self.verify_get(
            self.proxy.get_network_ip_availability,
            network_ip_availability.NetworkIPAvailability,
        )

    def test_network_ip_availabilities(self):
        self.verify_list(
            self.proxy.network_ip_availabilities,
            network_ip_availability.NetworkIPAvailability,
        )

    def test_pool_member_create_attrs(self):
        self.verify_create(
            self.proxy.create_pool_member,
            pool_member.PoolMember,
            method_kwargs={"pool": "test_id"},
            expected_kwargs={"pool_id": "test_id"},
        )


class TestNetworkPoolMember(TestNetworkProxy):
    def test_pool_member_delete(self):
        self.verify_delete(
            self.proxy.delete_pool_member,
            pool_member.PoolMember,
            ignore_missing=False,
            method_kwargs={"pool": "test_id"},
            expected_kwargs={"pool_id": "test_id"},
        )

    def test_pool_member_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_pool_member,
            pool_member.PoolMember,
            ignore_missing=True,
            method_kwargs={"pool": "test_id"},
            expected_kwargs={"pool_id": "test_id"},
        )

    def test_pool_member_find(self):
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_pool_member,
            method_args=["MEMBER", "POOL"],
            expected_args=[pool_member.PoolMember, "MEMBER"],
            expected_kwargs={"pool_id": "POOL", "ignore_missing": True},
        )

    def test_pool_member_get(self):
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_pool_member,
            method_args=["MEMBER", "POOL"],
            expected_args=[pool_member.PoolMember, "MEMBER"],
            expected_kwargs={"pool_id": "POOL"},
        )

    def test_pool_members(self):
        self.verify_list(
            self.proxy.pool_members,
            pool_member.PoolMember,
            method_args=["test_id"],
            expected_args=[],
            expected_kwargs={"pool_id": "test_id"},
        )

    def test_pool_member_update(self):
        self._verify(
            "openstack.network.v2._proxy.Proxy._update",
            self.proxy.update_pool_member,
            method_args=["MEMBER", "POOL"],
            expected_args=[pool_member.PoolMember, "MEMBER"],
            expected_kwargs={"pool_id": "POOL"},
        )


class TestNetworkPool(TestNetworkProxy):
    def test_pool_create_attrs(self):
        self.verify_create(self.proxy.create_pool, pool.Pool)

    def test_pool_delete(self):
        self.verify_delete(self.proxy.delete_pool, pool.Pool, False)

    def test_pool_delete_ignore(self):
        self.verify_delete(self.proxy.delete_pool, pool.Pool, True)

    def test_pool_find(self):
        self.verify_find(self.proxy.find_pool, pool.Pool)

    def test_pool_get(self):
        self.verify_get(self.proxy.get_pool, pool.Pool)

    def test_pools(self):
        self.verify_list(self.proxy.pools, pool.Pool)

    def test_pool_update(self):
        self.verify_update(self.proxy.update_pool, pool.Pool)

    def test_port_create_attrs(self):
        self.verify_create(self.proxy.create_port, port.Port)

    def test_port_delete(self):
        self.verify_delete(
            self.proxy.delete_port,
            port.Port,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_port_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_port,
            port.Port,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_port_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_port,
            port.Port,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_port_find(self):
        self.verify_find(self.proxy.find_port, port.Port)

    def test_port_get(self):
        self.verify_get(self.proxy.get_port, port.Port)

    def test_ports(self):
        self.verify_list(self.proxy.ports, port.Port)

    def test_port_update(self):
        self.verify_update(
            self.proxy.update_port,
            port.Port,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_port_update_if_revision(self):
        self.verify_update(
            self.proxy.update_port,
            port.Port,
            method_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
        )

    @mock.patch('openstack.network.v2._proxy.Proxy._bulk_create')
    def test_ports_create(self, bc):
        data = mock.sentinel

        self.proxy.create_ports(data)

        bc.assert_called_once_with(port.Port, data)


class TestNetworkQosBandwidth(TestNetworkProxy):
    def test_qos_bandwidth_limit_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_qos_bandwidth_limit_rule,
            qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_bandwidth_limit_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_qos_bandwidth_limit_rule,
            qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
            ignore_missing=False,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_bandwidth_limit_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_qos_bandwidth_limit_rule,
            qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
            ignore_missing=True,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_bandwidth_limit_rule_find(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_qos_bandwidth_limit_rule,
            method_args=['rule_id', policy],
            expected_args=[
                qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
                'rule_id',
            ],
            expected_kwargs={
                'ignore_missing': True,
                'qos_policy_id': QOS_POLICY_ID,
            },
        )

    def test_qos_bandwidth_limit_rule_get(self):
        self.verify_get(
            self.proxy.get_qos_bandwidth_limit_rule,
            qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_bandwidth_limit_rules(self):
        self.verify_list(
            self.proxy.qos_bandwidth_limit_rules,
            qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_bandwidth_limit_rule_update(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_qos_bandwidth_limit_rule,
            method_args=['rule_id', policy],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                qos_bandwidth_limit_rule.QoSBandwidthLimitRule,
                'rule_id',
            ],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'},
        )


class TestNetworkQosDscpMarking(TestNetworkProxy):
    def test_qos_dscp_marking_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_qos_dscp_marking_rule,
            qos_dscp_marking_rule.QoSDSCPMarkingRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_dscp_marking_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_qos_dscp_marking_rule,
            qos_dscp_marking_rule.QoSDSCPMarkingRule,
            ignore_missing=False,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_dscp_marking_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_qos_dscp_marking_rule,
            qos_dscp_marking_rule.QoSDSCPMarkingRule,
            ignore_missing=True,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_dscp_marking_rule_find(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_qos_dscp_marking_rule,
            method_args=['rule_id', policy],
            expected_args=[
                qos_dscp_marking_rule.QoSDSCPMarkingRule,
                'rule_id',
            ],
            expected_kwargs={
                'ignore_missing': True,
                'qos_policy_id': QOS_POLICY_ID,
            },
        )

    def test_qos_dscp_marking_rule_get(self):
        self.verify_get(
            self.proxy.get_qos_dscp_marking_rule,
            qos_dscp_marking_rule.QoSDSCPMarkingRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_dscp_marking_rules(self):
        self.verify_list(
            self.proxy.qos_dscp_marking_rules,
            qos_dscp_marking_rule.QoSDSCPMarkingRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_dscp_marking_rule_update(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_qos_dscp_marking_rule,
            method_args=['rule_id', policy],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                qos_dscp_marking_rule.QoSDSCPMarkingRule,
                'rule_id',
            ],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'},
        )


class TestNetworkQosMinimumBandwidth(TestNetworkProxy):
    def test_qos_minimum_bandwidth_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_qos_minimum_bandwidth_rule,
            qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_bandwidth_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_qos_minimum_bandwidth_rule,
            qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
            ignore_missing=False,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_bandwidth_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_qos_minimum_bandwidth_rule,
            qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
            ignore_missing=True,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_bandwidth_rule_find(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_qos_minimum_bandwidth_rule,
            method_args=['rule_id', policy],
            expected_args=[
                qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
                'rule_id',
            ],
            expected_kwargs={
                'ignore_missing': True,
                'qos_policy_id': QOS_POLICY_ID,
            },
        )

    def test_qos_minimum_bandwidth_rule_get(self):
        self.verify_get(
            self.proxy.get_qos_minimum_bandwidth_rule,
            qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_bandwidth_rules(self):
        self.verify_list(
            self.proxy.qos_minimum_bandwidth_rules,
            qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_bandwidth_rule_update(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_qos_minimum_bandwidth_rule,
            method_args=['rule_id', policy],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                qos_minimum_bandwidth_rule.QoSMinimumBandwidthRule,
                'rule_id',
            ],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'},
        )


class TestNetworkQosMinimumPacketRate(TestNetworkProxy):
    def test_qos_minimum_packet_rate_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_qos_minimum_packet_rate_rule,
            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_packet_rate_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_qos_minimum_packet_rate_rule,
            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
            ignore_missing=False,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_packet_rate_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_qos_minimum_packet_rate_rule,
            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
            ignore_missing=True,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_packet_rate_rule_find(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_qos_minimum_packet_rate_rule,
            method_args=['rule_id', policy],
            expected_args=[
                qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
                'rule_id',
            ],
            expected_kwargs={
                'ignore_missing': True,
                'qos_policy_id': QOS_POLICY_ID,
            },
        )

    def test_qos_minimum_packet_rate_rule_get(self):
        self.verify_get(
            self.proxy.get_qos_minimum_packet_rate_rule,
            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_packet_rate_rules(self):
        self.verify_list(
            self.proxy.qos_minimum_packet_rate_rules,
            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_minimum_packet_rate_rule_update(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_qos_minimum_packet_rate_rule,
            method_args=['rule_id', policy],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
                'rule_id',
            ],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'},
        )


class TestNetworkQosPacketRateLimitRule(TestNetworkProxy):
    def test_qos_packet_rate_limit_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_qos_packet_rate_limit_rule,
            qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_packet_rate_limit_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_qos_packet_rate_limit_rule,
            qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
            ignore_missing=False,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_packet_rate_limit_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_qos_packet_rate_limit_rule,
            qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
            ignore_missing=True,
            method_args=["resource_or_id", QOS_POLICY_ID],
            expected_args=["resource_or_id"],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_packet_rate_limit_rule_find(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_qos_packet_rate_limit_rule,
            method_args=['rule_id', policy],
            expected_args=[
                qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
                'rule_id',
            ],
            expected_kwargs={
                'ignore_missing': True,
                'qos_policy_id': QOS_POLICY_ID,
            },
        )

    def test_qos_packet_rate_limit_rule_get(self):
        self.verify_get(
            self.proxy.get_qos_packet_rate_limit_rule,
            qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_packet_rate_limit_rules(self):
        self.verify_list(
            self.proxy.qos_packet_rate_limit_rules,
            qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
            method_kwargs={'qos_policy': QOS_POLICY_ID},
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID},
        )

    def test_qos_packet_rate_limit_rule_update(self):
        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_qos_packet_rate_limit_rule,
            method_args=['rule_id', policy],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                qos_packet_rate_limit_rule.QoSPacketRateLimitRule,
                'rule_id',
            ],
            expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'},
        )


class TestNetworkQosRuleType(TestNetworkProxy):
    def test_qos_rule_type_find(self):
        self.verify_find(
            self.proxy.find_qos_rule_type, qos_rule_type.QoSRuleType
        )

    def test_qos_rule_type_get(self):
        self.verify_get(
            self.proxy.get_qos_rule_type, qos_rule_type.QoSRuleType
        )

    def test_qos_rule_types(self):
        self.verify_list(self.proxy.qos_rule_types, qos_rule_type.QoSRuleType)


class TestNetworkQuota(TestNetworkProxy):
    def test_quota_delete(self):
        self.verify_delete(self.proxy.delete_quota, quota.Quota, False)

    def test_quota_delete_ignore(self):
        self.verify_delete(self.proxy.delete_quota, quota.Quota, True)

    def test_quota_get(self):
        self.verify_get(self.proxy.get_quota, quota.Quota)

    @mock.patch.object(proxy_base.Proxy, "_get_resource")
    def test_quota_get_details(self, mock_get):
        fake_quota = mock.Mock(project_id='PROJECT')
        mock_get.return_value = fake_quota
        self._verify(
            "openstack.proxy.Proxy._get",
            self.proxy.get_quota,
            method_args=['QUOTA_ID'],
            method_kwargs={'details': True},
            expected_args=[quota.QuotaDetails],
            expected_kwargs={'project': fake_quota.id, 'requires_id': False},
        )
        mock_get.assert_called_once_with(quota.Quota, 'QUOTA_ID')

    @mock.patch.object(proxy_base.Proxy, "_get_resource")
    def test_quota_default_get(self, mock_get):
        fake_quota = mock.Mock(project_id='PROJECT')
        mock_get.return_value = fake_quota
        self._verify(
            "openstack.proxy.Proxy._get",
            self.proxy.get_quota_default,
            method_args=['QUOTA_ID'],
            expected_args=[quota.QuotaDefault],
            expected_kwargs={'project': fake_quota.id, 'requires_id': False},
        )
        mock_get.assert_called_once_with(quota.Quota, 'QUOTA_ID')

    def test_quotas(self):
        self.verify_list(self.proxy.quotas, quota.Quota)

    def test_quota_update(self):
        self.verify_update(self.proxy.update_quota, quota.Quota)


class TestNetworkRbacPolicy(TestNetworkProxy):
    def test_rbac_policy_create_attrs(self):
        self.verify_create(
            self.proxy.create_rbac_policy, rbac_policy.RBACPolicy
        )

    def test_rbac_policy_delete(self):
        self.verify_delete(
            self.proxy.delete_rbac_policy, rbac_policy.RBACPolicy, False
        )

    def test_rbac_policy_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_rbac_policy, rbac_policy.RBACPolicy, True
        )

    def test_rbac_policy_find(self):
        self.verify_find(self.proxy.find_rbac_policy, rbac_policy.RBACPolicy)

    def test_rbac_policy_get(self):
        self.verify_get(self.proxy.get_rbac_policy, rbac_policy.RBACPolicy)

    def test_rbac_policies(self):
        self.verify_list(self.proxy.rbac_policies, rbac_policy.RBACPolicy)

    def test_rbac_policy_update(self):
        self.verify_update(
            self.proxy.update_rbac_policy, rbac_policy.RBACPolicy
        )


class TestNetworkRouter(TestNetworkProxy):
    def test_router_create_attrs(self):
        self.verify_create(self.proxy.create_router, router.Router)

    def test_router_delete(self):
        self.verify_delete(
            self.proxy.delete_router,
            router.Router,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_router_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_router,
            router.Router,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_router_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_router,
            router.Router,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_router_find(self):
        self.verify_find(self.proxy.find_router, router.Router)

    def test_router_get(self):
        self.verify_get(self.proxy.get_router, router.Router)

    def test_routers(self):
        self.verify_list(self.proxy.routers, router.Router)

    def test_router_update(self):
        self.verify_update(
            self.proxy.update_router,
            router.Router,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_router_update_if_revision(self):
        self.verify_update(
            self.proxy.update_router,
            router.Router,
            method_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
        )

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'add_interface')
    def test_add_interface_to_router_with_port(
        self, mock_add_interface, mock_get
    ):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.add_interface",
            self.proxy.add_interface_to_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"port_id": "PORT"},
            expected_args=[self.proxy],
            expected_kwargs={"port_id": "PORT"},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'add_interface')
    def test_add_interface_to_router_with_subnet(
        self, mock_add_interface, mock_get
    ):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.add_interface",
            self.proxy.add_interface_to_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"subnet_id": "SUBNET"},
            expected_args=[self.proxy],
            expected_kwargs={"subnet_id": "SUBNET"},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'add_interface')
    def test_add_interface_to_router_with_advertise_host(
        self, mock_add_interface, mock_get
    ):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.add_interface",
            self.proxy.add_interface_to_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={
                "subnet_id": "SUBNET",
                "advertise_host": True,
            },
            expected_args=[self.proxy],
            expected_kwargs={
                "subnet_id": "SUBNET",
                "advertise_host": True,
            },
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'remove_interface')
    def test_remove_interface_from_router_with_port(
        self, mock_remove, mock_get
    ):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.remove_interface",
            self.proxy.remove_interface_from_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"port_id": "PORT"},
            expected_args=[self.proxy],
            expected_kwargs={"port_id": "PORT"},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'remove_interface')
    def test_remove_interface_from_router_with_subnet(
        self, mock_remove, mock_get
    ):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.remove_interface",
            self.proxy.remove_interface_from_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"subnet_id": "SUBNET"},
            expected_args=[self.proxy],
            expected_kwargs={"subnet_id": "SUBNET"},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'add_extra_routes')
    def test_add_extra_routes_to_router(self, mock_add_extra_routes, mock_get):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.add_extra_routes",
            self.proxy.add_extra_routes_to_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"body": {"router": {"routes": []}}},
            expected_args=[self.proxy],
            expected_kwargs={"body": {"router": {"routes": []}}},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'remove_extra_routes')
    def test_remove_extra_routes_from_router(
        self, mock_remove_extra_routes, mock_get
    ):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.remove_extra_routes",
            self.proxy.remove_extra_routes_from_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"body": {"router": {"routes": []}}},
            expected_args=[self.proxy],
            expected_kwargs={"body": {"router": {"routes": []}}},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'add_gateway')
    def test_add_gateway_to_router(self, mock_add, mock_get):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.add_gateway",
            self.proxy.add_gateway_to_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"foo": "bar"},
            expected_args=[self.proxy],
            expected_kwargs={"foo": "bar"},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'remove_gateway')
    def test_remove_gateway_from_router(self, mock_remove, mock_get):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.remove_gateway",
            self.proxy.remove_gateway_from_router,
            method_args=["FAKE_ROUTER"],
            method_kwargs={"foo": "bar"},
            expected_args=[self.proxy],
            expected_kwargs={"foo": "bar"},
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'add_external_gateways')
    def test_add_external_gateways(self, mock_add, mock_get):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.add_external_gateways",
            self.proxy.add_external_gateways,
            method_args=["FAKE_ROUTER", "bar"],
            expected_args=[self.proxy, "bar"],
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'update_external_gateways')
    def test_update_external_gateways(self, mock_remove, mock_get):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.update_external_gateways",
            self.proxy.update_external_gateways,
            method_args=["FAKE_ROUTER", "bar"],
            expected_args=[self.proxy, "bar"],
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    @mock.patch.object(router.Router, 'remove_external_gateways')
    def test_remove_external_gateways(self, mock_remove, mock_get):
        x_router = router.Router.new(id="ROUTER_ID")
        mock_get.return_value = x_router

        self._verify(
            "openstack.network.v2.router.Router.remove_external_gateways",
            self.proxy.remove_external_gateways,
            method_args=["FAKE_ROUTER", "bar"],
            expected_args=[self.proxy, "bar"],
        )
        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")

    def test_router_hosting_l3_agents_list(self):
        self.verify_list(
            self.proxy.routers_hosting_l3_agents,
            agent.RouterL3Agent,
            method_kwargs={'router': ROUTER_ID},
            expected_kwargs={'router_id': ROUTER_ID},
        )

    def test_agent_hosted_routers_list(self):
        self.verify_list(
            self.proxy.agent_hosted_routers,
            router.L3AgentRouter,
            method_kwargs={'agent': AGENT_ID},
            expected_kwargs={'agent_id': AGENT_ID},
        )


class TestNetworkFirewallGroup(TestNetworkProxy):
    def test_firewall_group_create_attrs(self):
        self.verify_create(
            self.proxy.create_firewall_group, firewall_group.FirewallGroup
        )

    def test_firewall_group_delete(self):
        self.verify_delete(
            self.proxy.delete_firewall_group,
            firewall_group.FirewallGroup,
            False,
        )

    def test_firewall_group_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_firewall_group,
            firewall_group.FirewallGroup,
            True,
        )

    def test_firewall_group_find(self):
        self.verify_find(
            self.proxy.find_firewall_group, firewall_group.FirewallGroup
        )

    def test_firewall_group_get(self):
        self.verify_get(
            self.proxy.get_firewall_group, firewall_group.FirewallGroup
        )

    def test_firewall_groups(self):
        self.verify_list(
            self.proxy.firewall_groups, firewall_group.FirewallGroup
        )

    def test_firewall_group_update(self):
        self.verify_update(
            self.proxy.update_firewall_group, firewall_group.FirewallGroup
        )


class TestNetworkPolicy(TestNetworkProxy):
    def test_firewall_policy_create_attrs(self):
        self.verify_create(
            self.proxy.create_firewall_policy, firewall_policy.FirewallPolicy
        )

    def test_firewall_policy_delete(self):
        self.verify_delete(
            self.proxy.delete_firewall_policy,
            firewall_policy.FirewallPolicy,
            False,
        )

    def test_firewall_policy_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_firewall_policy,
            firewall_policy.FirewallPolicy,
            True,
        )

    def test_firewall_policy_find(self):
        self.verify_find(
            self.proxy.find_firewall_policy, firewall_policy.FirewallPolicy
        )

    def test_firewall_policy_get(self):
        self.verify_get(
            self.proxy.get_firewall_policy, firewall_policy.FirewallPolicy
        )

    def test_firewall_policies(self):
        self.verify_list(
            self.proxy.firewall_policies, firewall_policy.FirewallPolicy
        )

    def test_firewall_policy_update(self):
        self.verify_update(
            self.proxy.update_firewall_policy, firewall_policy.FirewallPolicy
        )


class TestNetworkRule(TestNetworkProxy):
    def test_firewall_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_firewall_rule, firewall_rule.FirewallRule
        )

    def test_firewall_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_firewall_rule, firewall_rule.FirewallRule, False
        )

    def test_firewall_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_firewall_rule, firewall_rule.FirewallRule, True
        )

    def test_firewall_rule_find(self):
        self.verify_find(
            self.proxy.find_firewall_rule, firewall_rule.FirewallRule
        )

    def test_firewall_rule_get(self):
        self.verify_get(
            self.proxy.get_firewall_rule, firewall_rule.FirewallRule
        )

    def test_firewall_rules(self):
        self.verify_list(self.proxy.firewall_rules, firewall_rule.FirewallRule)

    def test_firewall_rule_update(self):
        self.verify_update(
            self.proxy.update_firewall_rule, firewall_rule.FirewallRule
        )


class TestNetworkNetworkSegment(TestNetworkProxy):
    def test_network_segment_range_create_attrs(self):
        self.verify_create(
            self.proxy.create_network_segment_range,
            network_segment_range.NetworkSegmentRange,
        )

    def test_network_segment_range_delete(self):
        self.verify_delete(
            self.proxy.delete_network_segment_range,
            network_segment_range.NetworkSegmentRange,
            False,
        )

    def test_network_segment_range_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_network_segment_range,
            network_segment_range.NetworkSegmentRange,
            True,
        )

    def test_network_segment_range_find(self):
        self.verify_find(
            self.proxy.find_network_segment_range,
            network_segment_range.NetworkSegmentRange,
        )

    def test_network_segment_range_get(self):
        self.verify_get(
            self.proxy.get_network_segment_range,
            network_segment_range.NetworkSegmentRange,
        )

    def test_network_segment_ranges(self):
        self.verify_list(
            self.proxy.network_segment_ranges,
            network_segment_range.NetworkSegmentRange,
        )

    def test_network_segment_range_update(self):
        self.verify_update(
            self.proxy.update_network_segment_range,
            network_segment_range.NetworkSegmentRange,
        )


class TestNetworkSecurityGroup(TestNetworkProxy):
    def test_security_group_create_attrs(self):
        self.verify_create(
            self.proxy.create_security_group, security_group.SecurityGroup
        )

    def test_security_group_delete(self):
        self.verify_delete(
            self.proxy.delete_security_group,
            security_group.SecurityGroup,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_security_group_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_security_group,
            security_group.SecurityGroup,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_security_group_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_security_group,
            security_group.SecurityGroup,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_security_group_find(self):
        self.verify_find(
            self.proxy.find_security_group, security_group.SecurityGroup
        )

    def test_security_group_get(self):
        self.verify_get(
            self.proxy.get_security_group, security_group.SecurityGroup
        )

    def test_security_groups(self):
        self.verify_list(
            self.proxy.security_groups, security_group.SecurityGroup
        )

    def test_security_group_update(self):
        self.verify_update(
            self.proxy.update_security_group,
            security_group.SecurityGroup,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_security_group_update_if_revision(self):
        self.verify_update(
            self.proxy.update_security_group,
            security_group.SecurityGroup,
            method_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': 42},
        )

    def test_security_group_rule_create_attrs(self):
        self.verify_create(
            self.proxy.create_security_group_rule,
            security_group_rule.SecurityGroupRule,
        )

    def test_security_group_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_security_group_rule,
            security_group_rule.SecurityGroupRule,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_security_group_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_security_group_rule,
            security_group_rule.SecurityGroupRule,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_security_group_rule_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_security_group_rule,
            security_group_rule.SecurityGroupRule,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_security_group_rule_find(self):
        self.verify_find(
            self.proxy.find_security_group_rule,
            security_group_rule.SecurityGroupRule,
        )

    def test_security_group_rule_get(self):
        self.verify_get(
            self.proxy.get_security_group_rule,
            security_group_rule.SecurityGroupRule,
        )

    def test_security_group_rules(self):
        self.verify_list(
            self.proxy.security_group_rules,
            security_group_rule.SecurityGroupRule,
        )

    @mock.patch('openstack.network.v2._proxy.Proxy._bulk_create')
    def test_security_group_rules_create(self, bc):
        data = mock.sentinel

        self.proxy.create_security_group_rules(data)

        bc.assert_called_once_with(security_group_rule.SecurityGroupRule, data)


class TestNetworkSecurityGroupsDefaultStatefulness(TestNetworkProxy):
    def test_create(self):
        self.verify_create(
            self.proxy.create_security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
        )

    def test_delete(self):
        self.verify_delete(
            self.proxy.delete_security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
            False,
        )

    def test_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
            True,
        )

    def test_find(self):
        self.verify_find(
            self.proxy.find_security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
        )

    def test_get(self):
        self.verify_get(
            self.proxy.get_security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
        )

    def test_list(self):
        self.verify_list(
            self.proxy.security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
        )

    def test_update(self):
        self.verify_update(
            self.proxy.update_security_groups_default_statefulness,
            security_groups_default_statefulness.SecurityGroupsDefaultStatefulness,
        )


class TestNetworkSegment(TestNetworkProxy):
    def test_segment_create_attrs(self):
        self.verify_create(self.proxy.create_segment, segment.Segment)

    def test_segment_delete(self):
        self.verify_delete(self.proxy.delete_segment, segment.Segment, False)

    def test_segment_delete_ignore(self):
        self.verify_delete(self.proxy.delete_segment, segment.Segment, True)

    def test_segment_find(self):
        self.verify_find(self.proxy.find_segment, segment.Segment)

    def test_segment_get(self):
        self.verify_get(self.proxy.get_segment, segment.Segment)

    def test_segments(self):
        self.verify_list(self.proxy.segments, segment.Segment)

    def test_segment_update(self):
        self.verify_update(self.proxy.update_segment, segment.Segment)


class TestNetworkSubnet(TestNetworkProxy):
    def test_subnet_create_attrs(self):
        self.verify_create(self.proxy.create_subnet, subnet.Subnet)

    def test_subnet_delete(self):
        self.verify_delete(
            self.proxy.delete_subnet,
            subnet.Subnet,
            False,
            expected_kwargs={'if_revision': None},
        )

    def test_subnet_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_subnet,
            subnet.Subnet,
            True,
            expected_kwargs={'if_revision': None},
        )

    def test_subnet_delete_if_revision(self):
        self.verify_delete(
            self.proxy.delete_subnet,
            subnet.Subnet,
            True,
            method_kwargs={'if_revision': 42},
            expected_kwargs={'if_revision': 42},
        )

    def test_subnet_find(self):
        self.verify_find(self.proxy.find_subnet, subnet.Subnet)

    def test_subnet_get(self):
        self.verify_get(self.proxy.get_subnet, subnet.Subnet)

    def test_subnets(self):
        self.verify_list(self.proxy.subnets, subnet.Subnet)

    def test_subnet_update(self):
        self.verify_update(
            self.proxy.update_subnet,
            subnet.Subnet,
            expected_kwargs={'x': 1, 'y': 2, 'z': 3, 'if_revision': None},
        )

    def test_subnet_pool_create_attrs(self):
        self.verify_create(
            self.proxy.create_subnet_pool, subnet_pool.SubnetPool
        )

    def test_subnet_pool_delete(self):
        self.verify_delete(
            self.proxy.delete_subnet_pool, subnet_pool.SubnetPool, False
        )

    def test_subnet_pool_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_subnet_pool, subnet_pool.SubnetPool, True
        )

    def test_subnet_pool_find(self):
        self.verify_find(self.proxy.find_subnet_pool, subnet_pool.SubnetPool)

    def test_subnet_pool_get(self):
        self.verify_get(self.proxy.get_subnet_pool, subnet_pool.SubnetPool)

    def test_subnet_pools(self):
        self.verify_list(self.proxy.subnet_pools, subnet_pool.SubnetPool)

    def test_subnet_pool_update(self):
        self.verify_update(
            self.proxy.update_subnet_pool, subnet_pool.SubnetPool
        )


class TestNetworkVpnEndpointGroup(TestNetworkProxy):
    def test_vpn_endpoint_group_create_attrs(self):
        self.verify_create(
            self.proxy.create_vpn_endpoint_group,
            vpn_endpoint_group.VpnEndpointGroup,
        )

    def test_vpn_endpoint_group_delete(self):
        self.verify_delete(
            self.proxy.delete_vpn_endpoint_group,
            vpn_endpoint_group.VpnEndpointGroup,
            False,
        )

    def test_vpn_endpoint_group_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_vpn_endpoint_group,
            vpn_endpoint_group.VpnEndpointGroup,
            True,
        )

    def test_vpn_endpoint_group_find(self):
        self.verify_find(
            self.proxy.find_vpn_endpoint_group,
            vpn_endpoint_group.VpnEndpointGroup,
        )

    def test_vpn_endpoint_group_get(self):
        self.verify_get(
            self.proxy.get_vpn_endpoint_group,
            vpn_endpoint_group.VpnEndpointGroup,
        )

    def test_vpn_endpoint_groups(self):
        self.verify_list(
            self.proxy.vpn_endpoint_groups, vpn_endpoint_group.VpnEndpointGroup
        )

    def test_vpn_endpoint_group_update(self):
        self.verify_update(
            self.proxy.update_vpn_endpoint_group,
            vpn_endpoint_group.VpnEndpointGroup,
        )


class TestNetworkVpnSiteConnection(TestNetworkProxy):
    def test_ipsec_site_connection_create_attrs(self):
        self.verify_create(
            self.proxy.create_vpn_ipsec_site_connection,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
        )

    def test_ipsec_site_connection_delete(self):
        self.verify_delete(
            self.proxy.delete_vpn_ipsec_site_connection,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
            False,
        )

    def test_ipsec_site_connection_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_vpn_ipsec_site_connection,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
            True,
        )

    def test_ipsec_site_connection_find(self):
        self.verify_find(
            self.proxy.find_vpn_ipsec_site_connection,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
        )

    def test_ipsec_site_connection_get(self):
        self.verify_get(
            self.proxy.get_vpn_ipsec_site_connection,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
        )

    def test_ipsec_site_connections(self):
        self.verify_list(
            self.proxy.vpn_ipsec_site_connections,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
        )

    def test_ipsec_site_connection_update(self):
        self.verify_update(
            self.proxy.update_vpn_ipsec_site_connection,
            vpn_ipsec_site_connection.VpnIPSecSiteConnection,
        )


class TestNetworkVpnIkePolicy(TestNetworkProxy):
    def test_ike_policy_create_attrs(self):
        self.verify_create(
            self.proxy.create_vpn_ike_policy, vpn_ike_policy.VpnIkePolicy
        )

    def test_ike_policy_delete(self):
        self.verify_delete(
            self.proxy.delete_vpn_ike_policy,
            vpn_ike_policy.VpnIkePolicy,
            False,
        )

    def test_ike_policy_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_vpn_ike_policy, vpn_ike_policy.VpnIkePolicy, True
        )

    def test_ike_policy_find(self):
        self.verify_find(
            self.proxy.find_vpn_ike_policy, vpn_ike_policy.VpnIkePolicy
        )

    def test_ike_policy_get(self):
        self.verify_get(
            self.proxy.get_vpn_ike_policy, vpn_ike_policy.VpnIkePolicy
        )

    def test_ike_policies(self):
        self.verify_list(
            self.proxy.vpn_ike_policies, vpn_ike_policy.VpnIkePolicy
        )

    def test_ike_policy_update(self):
        self.verify_update(
            self.proxy.update_vpn_ike_policy, vpn_ike_policy.VpnIkePolicy
        )


class TestNetworkVpnIpsecPolicy(TestNetworkProxy):
    def test_ipsec_policy_create_attrs(self):
        self.verify_create(
            self.proxy.create_vpn_ipsec_policy, vpn_ipsec_policy.VpnIpsecPolicy
        )

    def test_ipsec_policy_delete(self):
        self.verify_delete(
            self.proxy.delete_vpn_ipsec_policy,
            vpn_ipsec_policy.VpnIpsecPolicy,
            False,
        )

    def test_ipsec_policy_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_vpn_ipsec_policy,
            vpn_ipsec_policy.VpnIpsecPolicy,
            True,
        )

    def test_ipsec_policy_find(self):
        self.verify_find(
            self.proxy.find_vpn_ipsec_policy, vpn_ipsec_policy.VpnIpsecPolicy
        )

    def test_ipsec_policy_get(self):
        self.verify_get(
            self.proxy.get_vpn_ipsec_policy, vpn_ipsec_policy.VpnIpsecPolicy
        )

    def test_ipsec_policies(self):
        self.verify_list(
            self.proxy.vpn_ipsec_policies, vpn_ipsec_policy.VpnIpsecPolicy
        )

    def test_ipsec_policy_update(self):
        self.verify_update(
            self.proxy.update_vpn_ipsec_policy, vpn_ipsec_policy.VpnIpsecPolicy
        )


class TestNetworkVpnService(TestNetworkProxy):
    def test_vpn_service_create_attrs(self):
        self.verify_create(
            self.proxy.create_vpn_service, vpn_service.VpnService
        )

    def test_vpn_service_delete(self):
        self.verify_delete(
            self.proxy.delete_vpn_service, vpn_service.VpnService, False
        )

    def test_vpn_service_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_vpn_service, vpn_service.VpnService, True
        )

    def test_vpn_service_find(self):
        self.verify_find(self.proxy.find_vpn_service, vpn_service.VpnService)

    def test_vpn_service_get(self):
        self.verify_get(self.proxy.get_vpn_service, vpn_service.VpnService)

    def test_vpn_services(self):
        self.verify_list(self.proxy.vpn_services, vpn_service.VpnService)

    def test_vpn_service_update(self):
        self.verify_update(
            self.proxy.update_vpn_service, vpn_service.VpnService
        )


class TestNetworkServiceProvider(TestNetworkProxy):
    def test_service_provider(self):
        self.verify_list(
            self.proxy.service_providers, service_provider.ServiceProvider
        )


class TestNetworkAutoAllocatedTopology(TestNetworkProxy):
    def test_auto_allocated_topology_get(self):
        self.verify_get(
            self.proxy.get_auto_allocated_topology,
            auto_allocated_topology.AutoAllocatedTopology,
        )

    def test_auto_allocated_topology_delete(self):
        self.verify_delete(
            self.proxy.delete_auto_allocated_topology,
            auto_allocated_topology.AutoAllocatedTopology,
            False,
        )

    def test_auto_allocated_topology_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_auto_allocated_topology,
            auto_allocated_topology.AutoAllocatedTopology,
            True,
        )

    def test_validate_topology(self):
        self.verify_get(
            self.proxy.validate_auto_allocated_topology,
            auto_allocated_topology.ValidateTopology,
            method_args=[mock.sentinel.project_id],
            expected_args=[],
            expected_kwargs={
                "project": mock.sentinel.project_id,
                "requires_id": False,
            },
        )


class TestNetworkTags(TestNetworkProxy):
    def test_set_tags(self):
        x_network = network.Network.new(id='NETWORK_ID')
        self._verify(
            'openstack.network.v2.network.Network.set_tags',
            self.proxy.set_tags,
            method_args=[x_network, ['TAG1', 'TAG2']],
            expected_args=[self.proxy, ['TAG1', 'TAG2']],
            expected_result=mock.sentinel.result_set_tags,
        )

    @mock.patch('openstack.network.v2.network.Network.set_tags')
    def test_set_tags_resource_without_tag_suport(self, mock_set_tags):
        no_tag_resource = object()
        self.assertRaises(
            exceptions.InvalidRequest,
            self.proxy.set_tags,
            no_tag_resource,
            ['TAG1', 'TAG2'],
        )
        self.assertEqual(0, mock_set_tags.call_count)


class TestNetworkFloatingIp(TestNetworkProxy):
    def test_create_floating_ip_port_forwarding(self):
        self.verify_create(
            self.proxy.create_floating_ip_port_forwarding,
            port_forwarding.PortForwarding,
            method_kwargs={'floating_ip': FIP_ID},
            expected_kwargs={'floatingip_id': FIP_ID},
        )

    def test_delete_floating_ip_port_forwarding(self):
        self.verify_delete(
            self.proxy.delete_floating_ip_port_forwarding,
            port_forwarding.PortForwarding,
            ignore_missing=False,
            method_args=[FIP_ID, "resource_or_id"],
            expected_args=["resource_or_id"],
            expected_kwargs={'floatingip_id': FIP_ID},
        )

    def test_delete_floating_ip_port_forwarding_ignore(self):
        self.verify_delete(
            self.proxy.delete_floating_ip_port_forwarding,
            port_forwarding.PortForwarding,
            ignore_missing=True,
            method_args=[FIP_ID, "resource_or_id"],
            expected_args=["resource_or_id"],
            expected_kwargs={'floatingip_id': FIP_ID},
        )

    def test_find_floating_ip_port_forwarding(self):
        fip = floating_ip.FloatingIP.new(id=FIP_ID)
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_floating_ip_port_forwarding,
            method_args=[fip, 'port_forwarding_id'],
            expected_args=[
                port_forwarding.PortForwarding,
                'port_forwarding_id',
            ],
            expected_kwargs={'ignore_missing': True, 'floatingip_id': FIP_ID},
        )

    def test_get_floating_ip_port_forwarding(self):
        fip = floating_ip.FloatingIP.new(id=FIP_ID)
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_floating_ip_port_forwarding,
            method_args=[fip, 'port_forwarding_id'],
            expected_args=[
                port_forwarding.PortForwarding,
                'port_forwarding_id',
            ],
            expected_kwargs={'floatingip_id': FIP_ID},
        )

    def test_floating_ip_port_forwardings(self):
        self.verify_list(
            self.proxy.floating_ip_port_forwardings,
            port_forwarding.PortForwarding,
            method_kwargs={'floating_ip': FIP_ID},
            expected_kwargs={'floatingip_id': FIP_ID},
        )

    def test_update_floating_ip_port_forwarding(self):
        fip = floating_ip.FloatingIP.new(id=FIP_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_floating_ip_port_forwarding,
            method_args=[fip, 'port_forwarding_id'],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                port_forwarding.PortForwarding,
                'port_forwarding_id',
            ],
            expected_kwargs={'floatingip_id': FIP_ID, 'foo': 'bar'},
        )

    def test_create_l3_conntrack_helper(self):
        self.verify_create(
            self.proxy.create_conntrack_helper,
            l3_conntrack_helper.ConntrackHelper,
            method_kwargs={'router': ROUTER_ID},
            expected_kwargs={'router_id': ROUTER_ID},
        )

    def test_delete_l3_conntrack_helper(self):
        r = router.Router.new(id=ROUTER_ID)
        self.verify_delete(
            self.proxy.delete_conntrack_helper,
            l3_conntrack_helper.ConntrackHelper,
            ignore_missing=False,
            method_args=['resource_or_id', r],
            expected_args=['resource_or_id'],
            expected_kwargs={'router_id': ROUTER_ID},
        )

    def test_delete_l3_conntrack_helper_ignore(self):
        r = router.Router.new(id=ROUTER_ID)
        self.verify_delete(
            self.proxy.delete_conntrack_helper,
            l3_conntrack_helper.ConntrackHelper,
            ignore_missing=True,
            method_args=['resource_or_id', r],
            expected_args=['resource_or_id'],
            expected_kwargs={'router_id': ROUTER_ID},
        )

    def test_get_l3_conntrack_helper(self):
        r = router.Router.new(id=ROUTER_ID)
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_conntrack_helper,
            method_args=['conntrack_helper_id', r],
            expected_args=[
                l3_conntrack_helper.ConntrackHelper,
                'conntrack_helper_id',
            ],
            expected_kwargs={'router_id': ROUTER_ID},
        )

    def test_l3_conntrack_helpers(self):
        self.verify_list(
            self.proxy.conntrack_helpers,
            l3_conntrack_helper.ConntrackHelper,
            method_args=[ROUTER_ID],
            expected_args=[],
            expected_kwargs={'router_id': ROUTER_ID},
        )

    def test_update_l3_conntrack_helper(self):
        r = router.Router.new(id=ROUTER_ID)
        self._verify(
            'openstack.network.v2._proxy.Proxy._update',
            self.proxy.update_conntrack_helper,
            method_args=['conntrack_helper_id', r],
            method_kwargs={'foo': 'bar'},
            expected_args=[
                l3_conntrack_helper.ConntrackHelper,
                'conntrack_helper_id',
            ],
            expected_kwargs={'router_id': ROUTER_ID, 'foo': 'bar'},
        )


class TestNetworkNDPProxy(TestNetworkProxy):
    def test_ndp_proxy_create_attrs(self):
        self.verify_create(self.proxy.create_ndp_proxy, ndp_proxy.NDPProxy)

    def test_ndp_proxy_delete(self):
        self.verify_delete(
            self.proxy.delete_ndp_proxy, ndp_proxy.NDPProxy, False
        )

    def test_ndp_proxy_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_ndp_proxy, ndp_proxy.NDPProxy, True
        )

    def test_ndp_proxy_find(self):
        self.verify_find(self.proxy.find_ndp_proxy, ndp_proxy.NDPProxy)

    def test_ndp_proxy_get(self):
        self.verify_get(self.proxy.get_ndp_proxy, ndp_proxy.NDPProxy)

    def test_ndp_proxies(self):
        self.verify_list(self.proxy.ndp_proxies, ndp_proxy.NDPProxy)

    def test_ndp_proxy_update(self):
        self.verify_update(self.proxy.update_ndp_proxy, ndp_proxy.NDPProxy)


class TestNetworkBGP(TestNetworkProxy):
    def test_bgp_speaker_create(self):
        self.verify_create(
            self.proxy.create_bgp_speaker, bgp_speaker.BgpSpeaker
        )

    def test_bgp_speaker_delete(self):
        self.verify_delete(
            self.proxy.delete_bgp_speaker, bgp_speaker.BgpSpeaker, False
        )

    def test_bgp_speaker_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_bgp_speaker, bgp_speaker.BgpSpeaker, True
        )

    def test_bgp_speaker_find(self):
        self.verify_find(self.proxy.find_bgp_speaker, bgp_speaker.BgpSpeaker)

    def test_bgp_speaker_get(self):
        self.verify_get(self.proxy.get_bgp_speaker, bgp_speaker.BgpSpeaker)

    def test_bgp_speakers(self):
        self.verify_list(self.proxy.bgp_speakers, bgp_speaker.BgpSpeaker)

    def test_bgp_speaker_update(self):
        self.verify_update(
            self.proxy.update_bgp_speaker, bgp_speaker.BgpSpeaker
        )

    def test_bgp_peer_create(self):
        self.verify_create(self.proxy.create_bgp_peer, bgp_peer.BgpPeer)

    def test_bgp_peer_delete(self):
        self.verify_delete(self.proxy.delete_bgp_peer, bgp_peer.BgpPeer, False)

    def test_bgp_peer_delete_ignore(self):
        self.verify_delete(self.proxy.delete_bgp_peer, bgp_peer.BgpPeer, True)

    def test_bgp_peer_find(self):
        self.verify_find(self.proxy.find_bgp_peer, bgp_peer.BgpPeer)

    def test_bgp_peer_get(self):
        self.verify_get(self.proxy.get_bgp_peer, bgp_peer.BgpPeer)

    def test_bgp_peers(self):
        self.verify_list(self.proxy.bgp_peers, bgp_peer.BgpPeer)

    def test_bgp_peer_update(self):
        self.verify_update(self.proxy.update_bgp_peer, bgp_peer.BgpPeer)


class TestNetworkBGPVPN(TestNetworkProxy):
    NETWORK_ASSOCIATION = 'net-assoc-id' + uuid.uuid4().hex
    PORT_ASSOCIATION = 'port-assoc-id' + uuid.uuid4().hex
    ROUTER_ASSOCIATION = 'router-assoc-id' + uuid.uuid4().hex

    def test_bgpvpn_create(self):
        self.verify_create(self.proxy.create_bgpvpn, bgpvpn.BgpVpn)

    def test_bgpvpn_delete(self):
        self.verify_delete(self.proxy.delete_bgpvpn, bgpvpn.BgpVpn, False)

    def test_bgpvpn_delete_ignore(self):
        self.verify_delete(self.proxy.delete_bgpvpn, bgpvpn.BgpVpn, True)

    def test_bgpvpn_find(self):
        self.verify_find(self.proxy.find_bgpvpn, bgpvpn.BgpVpn)

    def test_bgpvpn_get(self):
        self.verify_get(self.proxy.get_bgpvpn, bgpvpn.BgpVpn)

    def test_bgpvpns(self):
        self.verify_list(self.proxy.bgpvpns, bgpvpn.BgpVpn)

    def test_bgpvpn_update(self):
        self.verify_update(self.proxy.update_bgpvpn, bgpvpn.BgpVpn)

    def test_bgpvpn_network_association_create(self):
        self.verify_create(
            self.proxy.create_bgpvpn_network_association,
            bgpvpn_network_association.BgpVpnNetworkAssociation,
            method_kwargs={'bgpvpn': BGPVPN_ID},
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_network_association_delete(self):
        self.verify_delete(
            self.proxy.delete_bgpvpn_network_association,
            bgpvpn_network_association.BgpVpnNetworkAssociation,
            False,
            method_args=[BGPVPN_ID, self.NETWORK_ASSOCIATION],
            expected_args=[self.NETWORK_ASSOCIATION],
            expected_kwargs={'ignore_missing': False, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_network_association_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_bgpvpn_network_association,
            bgpvpn_network_association.BgpVpnNetworkAssociation,
            True,
            method_args=[BGPVPN_ID, self.NETWORK_ASSOCIATION],
            expected_args=[self.NETWORK_ASSOCIATION],
            expected_kwargs={'ignore_missing': True, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_network_association_get(self):
        self.verify_get(
            self.proxy.get_bgpvpn_network_association,
            bgpvpn_network_association.BgpVpnNetworkAssociation,
            method_args=[BGPVPN_ID, self.NETWORK_ASSOCIATION],
            expected_args=[self.NETWORK_ASSOCIATION],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_network_associations(self):
        self.verify_list(
            self.proxy.bgpvpn_network_associations,
            bgpvpn_network_association.BgpVpnNetworkAssociation,
            method_args=[
                BGPVPN_ID,
            ],
            expected_args=[],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_association_create(self):
        self.verify_create(
            self.proxy.create_bgpvpn_port_association,
            bgpvpn_port_association.BgpVpnPortAssociation,
            method_kwargs={'bgpvpn': BGPVPN_ID},
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_association_delete(self):
        self.verify_delete(
            self.proxy.delete_bgpvpn_port_association,
            bgpvpn_port_association.BgpVpnPortAssociation,
            False,
            method_args=[BGPVPN_ID, self.PORT_ASSOCIATION],
            expected_args=[self.PORT_ASSOCIATION],
            expected_kwargs={'ignore_missing': False, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_association_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_bgpvpn_port_association,
            bgpvpn_port_association.BgpVpnPortAssociation,
            True,
            method_args=[BGPVPN_ID, self.PORT_ASSOCIATION],
            expected_args=[self.PORT_ASSOCIATION],
            expected_kwargs={'ignore_missing': True, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_association_find(self):
        self.verify_find(
            self.proxy.find_bgpvpn_port_association,
            bgpvpn_port_association.BgpVpnPortAssociation,
            method_args=[BGPVPN_ID],
            expected_args=['resource_name'],
            method_kwargs={'ignore_missing': True},
            expected_kwargs={'ignore_missing': True, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_association_get(self):
        self.verify_get(
            self.proxy.get_bgpvpn_port_association,
            bgpvpn_port_association.BgpVpnPortAssociation,
            method_args=[BGPVPN_ID, self.PORT_ASSOCIATION],
            expected_args=[self.PORT_ASSOCIATION],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_associations(self):
        self.verify_list(
            self.proxy.bgpvpn_port_associations,
            bgpvpn_port_association.BgpVpnPortAssociation,
            method_args=[
                BGPVPN_ID,
            ],
            expected_args=[],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_port_association_update(self):
        self.verify_update(
            self.proxy.update_bgpvpn_port_association,
            bgpvpn_port_association.BgpVpnPortAssociation,
            method_args=[BGPVPN_ID, self.PORT_ASSOCIATION],
            method_kwargs={},
            expected_args=[self.PORT_ASSOCIATION],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_router_association_create(self):
        self.verify_create(
            self.proxy.create_bgpvpn_router_association,
            bgpvpn_router_association.BgpVpnRouterAssociation,
            method_kwargs={'bgpvpn': BGPVPN_ID},
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_router_association_delete(self):
        self.verify_delete(
            self.proxy.delete_bgpvpn_router_association,
            bgpvpn_router_association.BgpVpnRouterAssociation,
            False,
            method_args=[BGPVPN_ID, self.ROUTER_ASSOCIATION],
            expected_args=[self.ROUTER_ASSOCIATION],
            expected_kwargs={'ignore_missing': False, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_router_association_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_bgpvpn_router_association,
            bgpvpn_router_association.BgpVpnRouterAssociation,
            True,
            method_args=[BGPVPN_ID, self.ROUTER_ASSOCIATION],
            expected_args=[self.ROUTER_ASSOCIATION],
            expected_kwargs={'ignore_missing': True, 'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_router_association_get(self):
        self.verify_get(
            self.proxy.get_bgpvpn_router_association,
            bgpvpn_router_association.BgpVpnRouterAssociation,
            method_args=[BGPVPN_ID, self.ROUTER_ASSOCIATION],
            expected_args=[self.ROUTER_ASSOCIATION],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_router_associations(self):
        self.verify_list(
            self.proxy.bgpvpn_router_associations,
            bgpvpn_router_association.BgpVpnRouterAssociation,
            method_args=[
                BGPVPN_ID,
            ],
            expected_args=[],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )

    def test_bgpvpn_router_association_update(self):
        self.verify_update(
            self.proxy.update_bgpvpn_router_association,
            bgpvpn_router_association.BgpVpnRouterAssociation,
            method_args=[BGPVPN_ID, self.ROUTER_ASSOCIATION],
            method_kwargs={},
            expected_args=[self.ROUTER_ASSOCIATION],
            expected_kwargs={'bgpvpn_id': BGPVPN_ID},
        )


class TestNetworkTapMirror(TestNetworkProxy):
    def test_create_tap_mirror(self):
        self.verify_create(self.proxy.create_tap_mirror, tap_mirror.TapMirror)

    def test_delete_tap_mirror(self):
        self.verify_delete(
            self.proxy.delete_tap_mirror, tap_mirror.TapMirror, False
        )

    def test_delete_tap_mirror_ignore(self):
        self.verify_delete(
            self.proxy.delete_tap_mirror, tap_mirror.TapMirror, True
        )

    def test_find_tap_mirror(self):
        self.verify_find(self.proxy.find_tap_mirror, tap_mirror.TapMirror)

    def test_get_tap_mirror(self):
        self.verify_get(self.proxy.get_tap_mirror, tap_mirror.TapMirror)

    def test_tap_mirrors(self):
        self.verify_list(self.proxy.tap_mirrors, tap_mirror.TapMirror)

    def test_update_tap_mirror(self):
        self.verify_update(self.proxy.update_tap_mirror, tap_mirror.TapMirror)


class TestNetworkPortBinding(TestNetworkProxy):
    @mock.patch.object(proxy_base.Proxy, '_get')
    def test_create_port_binding(self, mock_get):
        res_port = port.Port.new(id=PORT_ID)
        mock_get.return_value = res_port

        self.verify_create(
            self.proxy.create_port_binding,
            port_binding.PortBinding,
            method_kwargs={'port': PORT_ID},
            expected_kwargs={'port_id': PORT_ID},
        )

    @mock.patch('openstack.network.v2._proxy.Proxy.activate_port_binding')
    def test_activate_port_binding(self, activate_binding):
        data = mock.sentinel
        self.proxy.activate_port_binding(port_binding.PortBinding, data)
        activate_binding.assert_called_once_with(
            port_binding.PortBinding, data
        )

    @mock.patch.object(proxy_base.Proxy, '_get')
    def test_port_bindings(self, mock_get):
        res_port = port.Port.new(id=PORT_ID)
        mock_get.return_value = res_port

        self.verify_list(
            self.proxy.port_bindings,
            port_binding.PortBinding,
            method_kwargs={'port': PORT_ID},
            expected_kwargs={'port_id': PORT_ID},
        )

    @mock.patch('openstack.network.v2._proxy.Proxy.delete_port_binding')
    @mock.patch.object(proxy_base.Proxy, '_get')
    def test_delete_port_binding(self, mock_get, delete_port_binding):
        res_port = port.Port.new(id=PORT_ID)
        mock_get.return_value = res_port
        data = mock.sentinel

        self.proxy.delete_port_binding(port_binding.PortBinding, data)
        delete_port_binding.assert_called_once_with(
            port_binding.PortBinding, data
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter

from openstack import resource
from openstack import utils


class System(resource.Resource):
    resource_key = 'system'
    base_path = '/system'

    # capabilities

    def assign_role_to_user(
        self,
        session: adapter.Adapter,
        user: resource.Resource,
        role: resource.Resource,
    ) -> bool:
        """Assign role to user on system"""
        url = utils.urljoin(self.base_path, 'users', user.id, 'roles', role.id)
        resp = session.put(
            url,
        )
        if resp.status_code == 204:
            return True
        return False

    def validate_user_has_role(
        self,
        session: adapter.Adapter,
        user: resource.Resource,
        role: resource.Resource,
    ) -> bool:
        """Validates that a user has a role on a system"""
        url = utils.urljoin(self.base_path, 'users', user.id, 'roles', role.id)
        resp = session.head(
            url,
        )
        if resp.status_code == 204:
            return True
        return False

    def unassign_role_from_user(
        self,
        session: adapter.Adapter,
        user: resource.Resource,
        role: resource.Resource,
    ) -> bool:
        """Unassigns a role from a user on a system"""
        url = utils.urljoin(self.base_path, 'users', user.id, 'roles', role.id)
        resp = session.delete(
            url,
        )
        if resp.status_code == 204:
            return True
        return False

    def assign_role_to_group(
        self,
        session: adapter.Adapter,
        group: resource.Resource,
        role: resource.Resource,
    ) -> bool:
        """Assign role to group on system"""
        url = utils.urljoin(
            self.base_path, 'groups', group.id, 'roles', role.id
        )
        resp = session.put(
            url,
        )
        if resp.status_code == 204:
            return True
        return False

    def validate_group_has_role(
        self,
        session: adapter.Adapter,
        group: resource.Resource,
        role: resource.Resource,
    ) -> bool:
        """Validates that a group has a role on a system"""
        url = utils.urljoin(
            self.base_path, 'groups', group.id, 'roles', role.id
        )
        resp = session.head(
            url,
        )
        if resp.status_code == 204:
            return True
        return False

    def unassign_role_from_group(
        self,
        session: adapter.Adapter,
        group: resource.Resource,
        role: resource.Resource,
    ) -> bool:
        """Unassigns a role from a group on a system"""
        url = utils.urljoin(
            self.base_path, 'groups', group.id, 'roles', role.id
        )
        resp = session.delete(
            url,
        )
        if resp.status_code == 204:
            return True
        return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.clustering.v1 import _proxy
from openstack.clustering.v1 import action
from openstack.clustering.v1 import build_info
from openstack.clustering.v1 import cluster
from openstack.clustering.v1 import cluster_attr
from openstack.clustering.v1 import cluster_policy
from openstack.clustering.v1 import event
from openstack.clustering.v1 import node
from openstack.clustering.v1 import policy
from openstack.clustering.v1 import policy_type
from openstack.clustering.v1 import profile
from openstack.clustering.v1 import profile_type
from openstack.clustering.v1 import receiver
from openstack.clustering.v1 import service
from openstack import proxy as proxy_base
from openstack.tests.unit import test_proxy_base


class TestClusterProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_build_info_get(self):
        self.verify_get(
            self.proxy.get_build_info,
            build_info.BuildInfo,
            method_args=[],
            expected_kwargs={'requires_id': False},
        )

    def test_profile_types(self):
        self.verify_list(self.proxy.profile_types, profile_type.ProfileType)

    def test_profile_type_get(self):
        self.verify_get(self.proxy.get_profile_type, profile_type.ProfileType)

    def test_policy_types(self):
        self.verify_list(self.proxy.policy_types, policy_type.PolicyType)

    def test_policy_type_get(self):
        self.verify_get(self.proxy.get_policy_type, policy_type.PolicyType)

    def test_profile_create(self):
        self.verify_create(self.proxy.create_profile, profile.Profile)

    def test_profile_validate(self):
        self.verify_create(
            self.proxy.validate_profile, profile.ProfileValidate
        )

    def test_profile_delete(self):
        self.verify_delete(self.proxy.delete_profile, profile.Profile, False)

    def test_profile_delete_ignore(self):
        self.verify_delete(self.proxy.delete_profile, profile.Profile, True)

    def test_profile_find(self):
        self.verify_find(self.proxy.find_profile, profile.Profile)

    def test_profile_get(self):
        self.verify_get(self.proxy.get_profile, profile.Profile)

    def test_profiles(self):
        self.verify_list(
            self.proxy.profiles,
            profile.Profile,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    def test_profile_update(self):
        self.verify_update(self.proxy.update_profile, profile.Profile)

    def test_cluster_create(self):
        self.verify_create(self.proxy.create_cluster, cluster.Cluster)

    def test_cluster_delete(self):
        self.verify_delete(self.proxy.delete_cluster, cluster.Cluster, False)

    def test_cluster_delete_ignore(self):
        self.verify_delete(self.proxy.delete_cluster, cluster.Cluster, True)

    def test_cluster_force_delete(self):
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.force_delete",
            self.proxy.delete_cluster,
            method_args=["value", False, True],
            expected_args=[self.proxy],
        )

    def test_cluster_find(self):
        self.verify_find(self.proxy.find_cluster, cluster.Cluster)

    def test_cluster_get(self):
        self.verify_get(self.proxy.get_cluster, cluster.Cluster)

    def test_clusters(self):
        self.verify_list(
            self.proxy.clusters,
            cluster.Cluster,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    def test_cluster_update(self):
        self.verify_update(self.proxy.update_cluster, cluster.Cluster)

    def test_services(self):
        self.verify_list(self.proxy.services, service.Service)

    @mock.patch.object(proxy_base.Proxy, '_find')
    def test_resize_cluster(self, mock_find):
        mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
        mock_find.return_value = mock_cluster
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.resize",
            self.proxy.resize_cluster,
            method_args=["FAKE_CLUSTER"],
            method_kwargs={'k1': 'v1', 'k2': 'v2'},
            expected_args=[self.proxy],
            expected_kwargs={'k1': 'v1', 'k2': 'v2'},
        )
        mock_find.assert_called_once_with(
            cluster.Cluster, "FAKE_CLUSTER", ignore_missing=False
        )

    def test_resize_cluster_with_obj(self):
        mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.resize",
            self.proxy.resize_cluster,
            method_args=[mock_cluster],
            method_kwargs={'k1': 'v1', 'k2': 'v2'},
            expected_args=[self.proxy],
            expected_kwargs={'k1': 'v1', 'k2': 'v2'},
        )

    def test_collect_cluster_attrs(self):
        self.verify_list(
            self.proxy.collect_cluster_attrs,
            cluster_attr.ClusterAttr,
            method_args=['FAKE_ID', 'path.to.attr'],
            expected_args=[],
            expected_kwargs={'cluster_id': 'FAKE_ID', 'path': 'path.to.attr'},
        )

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_cluster_check(self, mock_get):
        mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
        mock_get.return_value = mock_cluster
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.check",
            self.proxy.check_cluster,
            method_args=["FAKE_CLUSTER"],
            expected_args=[self.proxy],
        )
        mock_get.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_cluster_recover(self, mock_get):
        mock_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')
        mock_get.return_value = mock_cluster
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.recover",
            self.proxy.recover_cluster,
            method_args=["FAKE_CLUSTER"],
            expected_args=[self.proxy],
        )
        mock_get.assert_called_once_with(cluster.Cluster, "FAKE_CLUSTER")

    def test_node_create(self):
        self.verify_create(self.proxy.create_node, node.Node)

    def test_node_delete(self):
        self.verify_delete(self.proxy.delete_node, node.Node, False)

    def test_node_delete_ignore(self):
        self.verify_delete(self.proxy.delete_node, node.Node, True)

    def test_node_force_delete(self):
        self._verify(
            "openstack.clustering.v1.node.Node.force_delete",
            self.proxy.delete_node,
            method_args=["value", False, True],
            expected_args=[self.proxy],
        )

    def test_node_find(self):
        self.verify_find(self.proxy.find_node, node.Node)

    def test_node_get(self):
        self.verify_get(self.proxy.get_node, node.Node)

    def test_node_get_with_details(self):
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_node,
            method_args=['NODE_ID'],
            method_kwargs={'details': True},
            expected_args=[node.NodeDetail],
            expected_kwargs={'node_id': 'NODE_ID', 'requires_id': False},
        )

    def test_nodes(self):
        self.verify_list(
            self.proxy.nodes,
            node.Node,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    def test_node_update(self):
        self.verify_update(self.proxy.update_node, node.Node)

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_node_check(self, mock_get):
        mock_node = node.Node.new(id='FAKE_NODE')
        mock_get.return_value = mock_node
        self._verify(
            "openstack.clustering.v1.node.Node.check",
            self.proxy.check_node,
            method_args=["FAKE_NODE"],
            expected_args=[self.proxy],
        )
        mock_get.assert_called_once_with(node.Node, "FAKE_NODE")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_node_recover(self, mock_get):
        mock_node = node.Node.new(id='FAKE_NODE')
        mock_get.return_value = mock_node
        self._verify(
            "openstack.clustering.v1.node.Node.recover",
            self.proxy.recover_node,
            method_args=["FAKE_NODE"],
            expected_args=[self.proxy],
        )
        mock_get.assert_called_once_with(node.Node, "FAKE_NODE")

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_node_adopt(self, mock_get):
        mock_node = node.Node.new()
        mock_get.return_value = mock_node
        self._verify(
            "openstack.clustering.v1.node.Node.adopt",
            self.proxy.adopt_node,
            method_kwargs={"preview": False, "foo": "bar"},
            expected_args=[self.proxy],
            expected_kwargs={"preview": False, "foo": "bar"},
        )

        mock_get.assert_called_once_with(node.Node, None)

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_node_adopt_preview(self, mock_get):
        mock_node = node.Node.new()
        mock_get.return_value = mock_node
        self._verify(
            "openstack.clustering.v1.node.Node.adopt",
            self.proxy.adopt_node,
            method_kwargs={"preview": True, "foo": "bar"},
            expected_args=[self.proxy],
            expected_kwargs={"preview": True, "foo": "bar"},
        )

        mock_get.assert_called_once_with(node.Node, None)

    def test_policy_create(self):
        self.verify_create(self.proxy.create_policy, policy.Policy)

    def test_policy_validate(self):
        self.verify_create(self.proxy.validate_policy, policy.PolicyValidate)

    def test_policy_delete(self):
        self.verify_delete(self.proxy.delete_policy, policy.Policy, False)

    def test_policy_delete_ignore(self):
        self.verify_delete(self.proxy.delete_policy, policy.Policy, True)

    def test_policy_find(self):
        self.verify_find(self.proxy.find_policy, policy.Policy)

    def test_policy_get(self):
        self.verify_get(self.proxy.get_policy, policy.Policy)

    def test_policies(self):
        self.verify_list(
            self.proxy.policies,
            policy.Policy,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    def test_policy_update(self):
        self.verify_update(self.proxy.update_policy, policy.Policy)

    def test_cluster_policies(self):
        self.verify_list(
            self.proxy.cluster_policies,
            cluster_policy.ClusterPolicy,
            method_args=["FAKE_CLUSTER"],
            expected_args=[],
            expected_kwargs={"cluster_id": "FAKE_CLUSTER"},
        )

    def test_get_cluster_policy(self):
        fake_policy = cluster_policy.ClusterPolicy.new(id="FAKE_POLICY")
        fake_cluster = cluster.Cluster.new(id='FAKE_CLUSTER')

        # ClusterPolicy object as input
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_cluster_policy,
            method_args=[fake_policy, "FAKE_CLUSTER"],
            expected_args=[cluster_policy.ClusterPolicy, fake_policy],
            expected_kwargs={'cluster_id': 'FAKE_CLUSTER'},
            expected_result=fake_policy,
        )

        # Policy ID as input
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_cluster_policy,
            method_args=["FAKE_POLICY", "FAKE_CLUSTER"],
            expected_args=[cluster_policy.ClusterPolicy, "FAKE_POLICY"],
            expected_kwargs={"cluster_id": "FAKE_CLUSTER"},
        )

        # Cluster object as input
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_cluster_policy,
            method_args=["FAKE_POLICY", fake_cluster],
            expected_args=[cluster_policy.ClusterPolicy, "FAKE_POLICY"],
            expected_kwargs={"cluster_id": fake_cluster},
        )

    def test_receiver_create(self):
        self.verify_create(self.proxy.create_receiver, receiver.Receiver)

    def test_receiver_update(self):
        self.verify_update(self.proxy.update_receiver, receiver.Receiver)

    def test_receiver_delete(self):
        self.verify_delete(
            self.proxy.delete_receiver, receiver.Receiver, False
        )

    def test_receiver_delete_ignore(self):
        self.verify_delete(self.proxy.delete_receiver, receiver.Receiver, True)

    def test_receiver_find(self):
        self.verify_find(self.proxy.find_receiver, receiver.Receiver)

    def test_receiver_get(self):
        self.verify_get(self.proxy.get_receiver, receiver.Receiver)

    def test_receivers(self):
        self.verify_list(
            self.proxy.receivers,
            receiver.Receiver,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    def test_action_get(self):
        self.verify_get(self.proxy.get_action, action.Action)

    def test_actions(self):
        self.verify_list(
            self.proxy.actions,
            action.Action,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    def test_action_update(self):
        self.verify_update(self.proxy.update_action, action.Action)

    def test_event_get(self):
        self.verify_get(self.proxy.get_event, event.Event)

    def test_events(self):
        self.verify_list(
            self.proxy.events,
            event.Event,
            method_kwargs={'limit': 2},
            expected_kwargs={'limit': 2},
        )

    @mock.patch("openstack.resource.wait_for_status")
    def test_wait_for(self, mock_wait):
        mock_resource = mock.Mock()
        mock_wait.return_value = mock_resource

        self.proxy.wait_for_status(mock_resource, 'ACTIVE')

        mock_wait.assert_called_once_with(
            self.proxy, mock_resource, 'ACTIVE', None, 2, None, 'status', None
        )

    @mock.patch("openstack.resource.wait_for_status")
    def test_wait_for_params(self, mock_wait):
        mock_resource = mock.Mock()
        mock_wait.return_value = mock_resource

        self.proxy.wait_for_status(mock_resource, 'ACTIVE', ['ERROR'], 1, 2)

        mock_wait.assert_called_once_with(
            self.proxy,
            mock_resource,
            'ACTIVE',
            ['ERROR'],
            1,
            2,
            'status',
            None,
        )

    @mock.patch("openstack.resource.wait_for_delete")
    def test_wait_for_delete(self, mock_wait):
        mock_resource = mock.Mock()
        mock_wait.return_value = mock_resource

        self.proxy.wait_for_delete(mock_resource)

        mock_wait.assert_called_once_with(
            self.proxy, mock_resource, 2, 120, None
        )

    @mock.patch("openstack.resource.wait_for_delete")
    def test_wait_for_delete_params(self, mock_wait):
        mock_resource = mock.Mock()
        mock_wait.return_value = mock_resource

        self.proxy.wait_for_delete(mock_resource, 1, 2)

        mock_wait.assert_called_once_with(
            self.proxy, mock_resource, 1, 2, None
        )

    def test_fetch_cluster_metadata(self):
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.fetch_metadata",
            self.proxy.fetch_cluster_metadata,
            method_args=["value"],
            expected_args=[self.proxy],
            expected_result=cluster.Cluster(id="value", metadata={}),
        )

    def test_set_cluster_metadata(self):
        kwargs = {"a": "1", "b": "2"}
        id = "an_id"
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.set_metadata",
            self.proxy.set_cluster_metadata,
            method_args=[id],
            method_kwargs=kwargs,
            method_result=cluster.Cluster.existing(id=id, metadata=kwargs),
            expected_args=[self.proxy],
            expected_kwargs={'metadata': kwargs},
            expected_result=cluster.Cluster.existing(id=id, metadata=kwargs),
        )

    def test_delete_cluster_metadata(self):
        self._verify(
            "openstack.clustering.v1.cluster.Cluster.delete_metadata_item",
            self.proxy.delete_cluster_metadata,
            expected_result=None,
            method_args=["value", ["key"]],
            expected_args=[self.proxy, "key"],
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from typing import Any, IO, cast

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.asymmetric import utils
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization

from openstack import exceptions
from openstack.image.iterable_chunked_file import IterableChunkedFile

HASH_METHODS = {
    'SHA-224': hashes.SHA224(),
    'SHA-256': hashes.SHA256(),
    'SHA-384': hashes.SHA384(),
    'SHA-512': hashes.SHA512(),
}


class ImageSigner:
    """Image file signature generator.

    Generates signatures for files using a specified private key file.
    """

    def __init__(
        self, hash_method: str = 'SHA-256', padding_method: str = 'RSA-PSS'
    ) -> None:
        padding_types = {
            'RSA-PSS': padding.PSS(
                mgf=padding.MGF1(HASH_METHODS[hash_method]),
                salt_length=padding.PSS.MAX_LENGTH,
            )
        }
        # informational attributes
        self.hash_method = hash_method
        self.padding_method = padding_method
        # runtime objects
        self.private_key: Any = None
        self.hash = HASH_METHODS[hash_method]
        self.hasher = hashes.Hash(self.hash, default_backend())
        self.padding = padding_types[padding_method]

    def load_private_key(
        self, file_path: str, password: bytes | None = None
    ) -> None:
        with open(file_path, 'rb') as key_file:
            self.private_key = serialization.load_pem_private_key(
                key_file.read(), password=password, backend=default_backend()
            )

    def generate_signature(self, file_obj: IO[bytes]) -> bytes:
        if not self.private_key:
            raise exceptions.SDKException("private_key not set")

        file_obj.seek(0)
        chunked_file = IterableChunkedFile(file_obj)
        for chunk in chunked_file:
            self.hasher.update(chunk)
        file_obj.seek(0)
        digest = self.hasher.finalize()
        signature = self.private_key.sign(
            digest, self.padding, utils.Prehashed(self.hash)
        )
        return cast(bytes, signature)



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import firewall_policy
from openstack.tests.unit import base


EXAMPLE = {
    'description': '1',
    'name': '2',
    'firewall_rules': [
        'a30b0ec2-a468-4b1c-8dbf-928ded2a57a8',
        '8d562e98-24f3-46e1-bbf3-d9347c0a67ee',
    ],
    'shared': True,
    'project_id': '4',
}


class TestFirewallPolicy(base.TestCase):
    def test_basic(self):
        sot = firewall_policy.FirewallPolicy()
        self.assertEqual('firewall_policy', sot.resource_key)
        self.assertEqual('firewall_policies', sot.resources_key)
        self.assertEqual('/fwaas/firewall_policies', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = firewall_policy.FirewallPolicy(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['firewall_rules'], sot.firewall_rules)
        self.assertEqual(EXAMPLE['shared'], sot.shared)
        self.assertEqual(list, type(sot.firewall_rules))
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack.shared_file_system.v2 import share as _share
from openstack.tests.functional.shared_file_system.v2 import base


class ShareTest(base.BaseSharedFileSystemTest):
    def setUp(self):
        super().setUp()

        self.SHARE_NAME = self.getUniqueString()
        my_share = self.create_share(
            name=self.SHARE_NAME,
            size=2,
            share_type="dhss_false",
            share_protocol='NFS',
            description=None,
        )
        self.SHARE_ID = my_share.id
        self.SHARE_SIZE = my_share.size

    def test_get(self):
        sot = self.user_cloud.share.get_share(self.SHARE_ID)
        assert isinstance(sot, _share.Share)
        self.assertEqual(self.SHARE_ID, sot.id)

    def test_find(self):
        sot = self.user_cloud.share.find_share(name_or_id=self.SHARE_NAME)
        assert isinstance(sot, _share.Share)
        self.assertEqual(self.SHARE_ID, sot.id)

    def test_list_share(self):
        shares = self.user_cloud.share.shares(details=False)
        self.assertGreater(len(list(shares)), 0)
        for share in shares:
            for attribute in ('id', 'name', 'created_at', 'updated_at'):
                self.assertTrue(hasattr(share, attribute))

    def test_update(self):
        updated_share = self.user_cloud.share.update_share(
            self.SHARE_ID, display_description='updated share'
        )
        get_updated_share = self.user_cloud.share.get_share(updated_share.id)
        self.assertEqual('updated share', get_updated_share.description)

    def test_revert_share_to_snapshot(self):
        my_share_snapshot = self.create_share_snapshot(share_id=self.SHARE_ID)
        self.SHARE_SNAPSHOT_ID = my_share_snapshot.id

        self.user_cloud.share.revert_share_to_snapshot(
            self.SHARE_ID, self.SHARE_SNAPSHOT_ID
        )
        get_reverted_share = self.user_cloud.share.get_share(self.SHARE_ID)
        self.user_cloud.share.wait_for_status(
            get_reverted_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.assertIsNotNone(get_reverted_share.id)

    def test_resize_share_larger(self):
        larger_size = 3
        self.user_cloud.share.resize_share(self.SHARE_ID, larger_size)

        get_resized_share = self.user_cloud.share.get_share(self.SHARE_ID)

        self.user_cloud.share.wait_for_status(
            get_resized_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.assertEqual(larger_size, get_resized_share.size)

    def test_resize_share_smaller(self):
        # Resize to 3 GiB
        smaller_size = 1

        self.user_cloud.share.resize_share(self.SHARE_ID, smaller_size)

        get_resized_share = self.user_cloud.share.get_share(self.SHARE_ID)

        self.user_cloud.share.wait_for_status(
            get_resized_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.assertEqual(smaller_size, get_resized_share.size)

    def test_resize_share_larger_no_extend(self):
        larger_size = 3

        self.user_cloud.share.resize_share(
            self.SHARE_ID, larger_size, no_extend=True
        )

        get_resized_share = self.user_cloud.share.get_share(self.SHARE_ID)

        self.user_cloud.share.wait_for_status(
            get_resized_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )

        # Assert that no change was made.
        self.assertEqual(self.SHARE_SIZE, get_resized_share.size)

    def test_resize_share_smaller_no_shrink(self):
        smaller_size = 1

        self.user_cloud.share.resize_share(
            self.SHARE_ID, smaller_size, no_shrink=True
        )

        get_resized_share = self.user_cloud.share.get_share(self.SHARE_ID)

        self.user_cloud.share.wait_for_status(
            get_resized_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )

        # Assert that no change was made.
        self.assertEqual(self.SHARE_SIZE, get_resized_share.size)

    def test_resize_share_with_force(self):
        """Test that extend with force works as expected."""
        # Resize to 3 GiB
        larger_size = 3
        self.operator_cloud.share.resize_share(
            self.SHARE_ID, larger_size, force=True
        )

        get_resized_share = self.user_cloud.share.get_share(self.SHARE_ID)

        self.user_cloud.share.wait_for_status(
            get_resized_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.assertEqual(larger_size, get_resized_share.size)

    def test_force_delete(self):
        sot = self.user_cloud.share.get_share(self.SHARE_ID)
        self.operator_cloud.share.delete_share(self.SHARE_ID, force=True)
        self.operator_cloud.share.wait_for_delete(sot)


class ManageUnmanageShareTest(base.BaseSharedFileSystemTest):
    def setUp(self):
        super().setUp()

        self.NEW_SHARE = self.create_share(
            share_proto="NFS",
            name="accounting_p8787",
            size=2,
        )
        self.SHARE_ID = self.NEW_SHARE.id

        self.export_locations = self.operator_cloud.share.export_locations(
            self.SHARE_ID
        )
        export_paths = [export['path'] for export in self.export_locations]
        self.export_path = export_paths[0]

        self.share_host = self.operator_cloud.share.get_share(self.SHARE_ID)[
            'host'
        ]

    def test_manage_and_unmanage_share(self):
        self.operator_cloud.share.unmanage_share(self.SHARE_ID)

        self.operator_cloud.shared_file_system.wait_for_delete(
            self.NEW_SHARE, interval=2, wait=self._wait_for_timeout
        )

        try:
            self.operator_cloud.share.get_share(self.SHARE_ID)
        except exceptions.NotFoundException:
            pass

        managed_share = self.operator_cloud.share.manage_share(
            self.NEW_SHARE.share_protocol, self.export_path, self.share_host
        )

        self.operator_cloud.share.wait_for_status(
            managed_share,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )

        self.assertEqual(
            self.NEW_SHARE.share_protocol, managed_share.share_protocol
        )

        managed_host = self.operator_cloud.share.get_share(managed_share.id)[
            'host'
        ]

        self.assertEqual(self.share_host, managed_host)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_router
----------------------------------

Functional tests for router methods.
"""

import ipaddress

from openstack import exceptions
from openstack.tests.functional import base


EXPECTED_TOPLEVEL_FIELDS = (
    'id',
    'name',
    'is_admin_state_up',
    'external_gateway_info',
    'project_id',
    'routes',
    'status',
)

EXPECTED_GW_INFO_FIELDS = ('network_id', 'enable_snat', 'external_fixed_ips')


class TestRouter(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()
        if not self.operator_cloud:
            self.skipTest("Operator cloud required for this test")
        if not self.operator_cloud.has_service('network'):
            self.skipTest('Network service not supported by cloud')

        self.router_prefix = self.getUniqueString('router')
        self.network_prefix = self.getUniqueString('network')
        self.subnet_prefix = self.getUniqueString('subnet')

        # NOTE(Shrews): Order matters!
        self.addCleanup(self._cleanup_networks)
        self.addCleanup(self._cleanup_subnets)
        self.addCleanup(self._cleanup_routers)

    def _cleanup_routers(self):
        exception_list = list()
        for router in self.operator_cloud.list_routers():
            if router['name'].startswith(self.router_prefix):
                try:
                    self.operator_cloud.delete_router(router['name'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            raise exceptions.SDKException('\n'.join(exception_list))

    def _cleanup_networks(self):
        exception_list = list()
        for network in self.operator_cloud.list_networks():
            if network['name'].startswith(self.network_prefix):
                try:
                    self.operator_cloud.delete_network(network['name'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            raise exceptions.SDKException('\n'.join(exception_list))

    def _cleanup_subnets(self):
        exception_list = list()
        for subnet in self.operator_cloud.list_subnets():
            if subnet['name'].startswith(self.subnet_prefix):
                try:
                    self.operator_cloud.delete_subnet(subnet['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            raise exceptions.SDKException('\n'.join(exception_list))

    def test_create_router_basic(self):
        net1_name = self.network_prefix + '_net1'
        net1 = self.operator_cloud.create_network(
            name=net1_name, external=True
        )

        router_name = self.router_prefix + '_create_basic'
        router = self.operator_cloud.create_router(
            name=router_name,
            admin_state_up=True,
            ext_gateway_net_id=net1['id'],
        )

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, router)

        ext_gw_info = router['external_gateway_info']
        for field in EXPECTED_GW_INFO_FIELDS:
            self.assertIn(field, ext_gw_info)

        self.assertEqual(router_name, router['name'])
        self.assertEqual('ACTIVE', router['status'])
        self.assertEqual(net1['id'], ext_gw_info['network_id'])
        self.assertTrue(ext_gw_info['enable_snat'])

    def test_create_router_project(self):
        project = self.operator_cloud.get_project('demo')
        self.assertIsNotNone(project)
        proj_id = project['id']
        net1_name = self.network_prefix + '_net1'
        net1 = self.operator_cloud.create_network(
            name=net1_name, external=True, project_id=proj_id
        )

        router_name = self.router_prefix + '_create_project'
        router = self.operator_cloud.create_router(
            name=router_name,
            admin_state_up=True,
            ext_gateway_net_id=net1['id'],
            project_id=proj_id,
        )

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, router)

        ext_gw_info = router['external_gateway_info']
        for field in EXPECTED_GW_INFO_FIELDS:
            self.assertIn(field, ext_gw_info)

        self.assertEqual(router_name, router['name'])
        self.assertEqual('ACTIVE', router['status'])
        self.assertEqual(proj_id, router['project_id'])
        self.assertEqual(net1['id'], ext_gw_info['network_id'])
        self.assertTrue(ext_gw_info['enable_snat'])

    def _create_and_verify_advanced_router(
        self, external_cidr, external_gateway_ip=None
    ):
        # external_cidr must be passed in as unicode (u'')
        # NOTE(Shrews): The arguments are needed because these tests
        # will run in parallel and we want to make sure that each test
        # is using different resources to prevent race conditions.
        net1_name = self.network_prefix + '_net1'
        sub1_name = self.subnet_prefix + '_sub1'
        net1 = self.operator_cloud.create_network(
            name=net1_name, external=True
        )
        sub1 = self.operator_cloud.create_subnet(
            net1['id'],
            external_cidr,
            subnet_name=sub1_name,
            gateway_ip=external_gateway_ip,
        )

        ip_net = ipaddress.IPv4Network(external_cidr)
        last_ip = str(list(ip_net.hosts())[-1])

        router_name = self.router_prefix + '_create_advanced'
        router = self.operator_cloud.create_router(
            name=router_name,
            admin_state_up=False,
            ext_gateway_net_id=net1['id'],
            enable_snat=False,
            ext_fixed_ips=[{'subnet_id': sub1['id'], 'ip_address': last_ip}],
        )

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, router)

        ext_gw_info = router['external_gateway_info']
        for field in EXPECTED_GW_INFO_FIELDS:
            self.assertIn(field, ext_gw_info)

        self.assertEqual(router_name, router['name'])
        self.assertEqual('ACTIVE', router['status'])
        self.assertFalse(router['admin_state_up'])

        self.assertEqual(1, len(ext_gw_info['external_fixed_ips']))
        self.assertEqual(
            sub1['id'], ext_gw_info['external_fixed_ips'][0]['subnet_id']
        )
        self.assertEqual(
            last_ip, ext_gw_info['external_fixed_ips'][0]['ip_address']
        )

        return router

    def test_create_router_advanced(self):
        self._create_and_verify_advanced_router(external_cidr='10.2.2.0/24')

    def test_add_remove_router_interface(self):
        router = self._create_and_verify_advanced_router(
            external_cidr='10.3.3.0/24'
        )
        net_name = self.network_prefix + '_intnet1'
        sub_name = self.subnet_prefix + '_intsub1'
        net = self.operator_cloud.create_network(name=net_name)
        sub = self.operator_cloud.create_subnet(
            net['id'],
            '10.4.4.0/24',
            subnet_name=sub_name,
            gateway_ip='10.4.4.1',
        )

        iface = self.operator_cloud.add_router_interface(
            router, subnet_id=sub['id']
        )
        self.assertIsNone(
            self.operator_cloud.remove_router_interface(
                router, subnet_id=sub['id']
            )
        )

        # Test return values *after* the interface is detached so the
        # resources we've created can be cleaned up if these asserts fail.
        self.assertIsNotNone(iface)
        for key in ('id', 'subnet_id', 'port_id', 'project_id'):
            self.assertIn(key, iface)
        self.assertEqual(router['id'], iface['id'])
        self.assertEqual(sub['id'], iface['subnet_id'])

    def test_list_router_interfaces(self):
        router = self._create_and_verify_advanced_router(
            external_cidr='10.5.5.0/24'
        )
        net_name = self.network_prefix + '_intnet1'
        sub_name = self.subnet_prefix + '_intsub1'
        net = self.operator_cloud.create_network(name=net_name)
        sub = self.operator_cloud.create_subnet(
            net['id'],
            '10.6.6.0/24',
            subnet_name=sub_name,
            gateway_ip='10.6.6.1',
        )

        iface = self.operator_cloud.add_router_interface(
            router, subnet_id=sub['id']
        )
        all_ifaces = self.operator_cloud.list_router_interfaces(router)
        int_ifaces = self.operator_cloud.list_router_interfaces(
            router, interface_type='internal'
        )
        ext_ifaces = self.operator_cloud.list_router_interfaces(
            router, interface_type='external'
        )
        self.assertIsNone(
            self.operator_cloud.remove_router_interface(
                router, subnet_id=sub['id']
            )
        )

        # Test return values *after* the interface is detached so the
        # resources we've created can be cleaned up if these asserts fail.
        self.assertIsNotNone(iface)
        self.assertEqual(2, len(all_ifaces))
        self.assertEqual(1, len(int_ifaces))
        self.assertEqual(1, len(ext_ifaces))

        ext_fixed_ips = router['external_gateway_info']['external_fixed_ips']
        self.assertEqual(
            ext_fixed_ips[0]['subnet_id'],
            ext_ifaces[0]['fixed_ips'][0]['subnet_id'],
        )
        self.assertEqual(sub['id'], int_ifaces[0]['fixed_ips'][0]['subnet_id'])

    def test_update_router_name(self):
        router = self._create_and_verify_advanced_router(
            external_cidr='10.7.7.0/24'
        )

        new_name = self.router_prefix + '_update_name'
        updated = self.operator_cloud.update_router(
            router['id'], name=new_name
        )
        self.assertIsNotNone(updated)

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, updated)

        # Name is the only change we expect
        self.assertEqual(new_name, updated['name'])

        # Validate nothing else changed
        self.assertEqual(router['status'], updated['status'])
        self.assertEqual(router['admin_state_up'], updated['admin_state_up'])
        self.assertEqual(
            router['external_gateway_info'], updated['external_gateway_info']
        )

    def test_update_router_routes(self):
        router = self._create_and_verify_advanced_router(
            external_cidr='10.7.7.0/24'
        )

        routes = [{"destination": "10.7.7.0/24", "nexthop": "10.7.7.99"}]

        updated = self.operator_cloud.update_router(
            router['id'], routes=routes
        )
        self.assertIsNotNone(updated)

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, updated)

        # Name is the only change we expect
        self.assertEqual(routes, updated['routes'])

        # Validate nothing else changed
        self.assertEqual(router['status'], updated['status'])
        self.assertEqual(router['admin_state_up'], updated['admin_state_up'])
        self.assertEqual(
            router['external_gateway_info'], updated['external_gateway_info']
        )

    def test_update_router_admin_state(self):
        router = self._create_and_verify_advanced_router(
            external_cidr='10.8.8.0/24'
        )

        updated = self.operator_cloud.update_router(
            router['id'], admin_state_up=True
        )
        self.assertIsNotNone(updated)

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, updated)

        # admin_state_up is the only change we expect
        self.assertTrue(updated['admin_state_up'])
        self.assertNotEqual(
            router['admin_state_up'], updated['admin_state_up']
        )

        # Validate nothing else changed
        self.assertEqual(router['status'], updated['status'])
        self.assertEqual(router['name'], updated['name'])
        self.assertEqual(
            router['external_gateway_info'], updated['external_gateway_info']
        )

    def test_update_router_ext_gw_info(self):
        router = self._create_and_verify_advanced_router(
            external_cidr='10.9.9.0/24'
        )

        # create a new subnet
        existing_net_id = router['external_gateway_info']['network_id']
        sub_name = self.subnet_prefix + '_update'
        sub = self.operator_cloud.create_subnet(
            existing_net_id,
            '10.10.10.0/24',
            subnet_name=sub_name,
            gateway_ip='10.10.10.1',
        )

        updated = self.operator_cloud.update_router(
            router['id'],
            ext_gateway_net_id=existing_net_id,
            ext_fixed_ips=[
                {'subnet_id': sub['id'], 'ip_address': '10.10.10.77'}
            ],
        )
        self.assertIsNotNone(updated)

        for field in EXPECTED_TOPLEVEL_FIELDS:
            self.assertIn(field, updated)

        # external_gateway_info is the only change we expect
        ext_gw_info = updated['external_gateway_info']
        self.assertEqual(1, len(ext_gw_info['external_fixed_ips']))
        self.assertEqual(
            sub['id'], ext_gw_info['external_fixed_ips'][0]['subnet_id']
        )
        self.assertEqual(
            '10.10.10.77', ext_gw_info['external_fixed_ips'][0]['ip_address']
        )

        # Validate nothing else changed
        self.assertEqual(router['status'], updated['status'])
        self.assertEqual(router['name'], updated['name'])
        self.assertEqual(router['admin_state_up'], updated['admin_state_up'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any

from keystoneauth1 import adapter

from openstack import resource
from openstack import utils


class ShareAccessRule(resource.Resource):
    resource_key = "access"
    resources_key = "access_list"
    base_path = "/share-access-rules"

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = False
    allow_delete = True
    allow_list = True
    allow_head = False

    _query_mapping = resource.QueryParameters("share_id")

    # Restricted access rules became available in 2.82
    _max_microversion = '2.82'

    create_opts = resource.CreateOpts(
        request_key='allow_access', response_key='access'
    )

    #: Properties
    #: The access credential of the entity granted share access.
    access_key = resource.Body("access_key", type=str)
    #: The access level to the share.
    access_level = resource.Body("access_level", type=str)
    #: The object of the access rule.
    access_list = resource.Body("access_list", type=str)
    #: The value that defines the access.
    access_to = resource.Body("access_to", type=str)
    #: The access rule type.
    access_type = resource.Body("access_type", type=str)
    #: The date and time stamp when the resource was created within the
    #: services's database.
    created_at = resource.Body("created_at", type=str)
    #: One or more access rule metadata key and value pairs as a dictionary
    #: of strings.
    metadata = resource.Body("metadata", type=dict)
    #: The UUID of the share to which you are granted or denied access.
    share_id = resource.Body("share_id", type=str)
    #: The state of the access rule.
    state = resource.Body("state", type=str)
    #: The date and time stamp when the resource was last updated within
    #: the services's database.
    updated_at = resource.Body("updated_at", type=str)
    #: Whether the visibility of some sensitive fields is restricted or not
    lock_visibility = resource.Body("lock_visibility", type=bool)
    #: Whether the deletion of the access rule should be restricted or not
    lock_deletion = resource.Body("lock_deletion", type=bool)
    #: Reason for placing the loc
    lock_reason = resource.Body("lock_reason", type=bool)

    def _action(
        self,
        session: adapter.Adapter,
        body: dict[str, Any],
        url: str,
        microversion: str | None = None,
    ) -> Any:
        headers = {'Accept': ''}

        if microversion is None:
            microversion = self._get_microversion(session)

        return session.post(
            url, json=body, headers=headers, microversion=microversion
        )

    def delete(
        self,
        session: adapter.Adapter,
        error_message: str | None = None,
        *,
        microversion: str | None = None,
        unrestrict: bool = False,
        **kwargs: Any,
    ) -> Any:
        body: dict[str, Any] = {'deny_access': {'access_id': self.id}}
        if unrestrict:
            body['deny_access']['unrestrict'] = True
        url = utils.urljoin("/shares", self.share_id, "action")
        response = self._action(session, body, url)
        self._translate_response(response)
        return response
